import json
import os
from datetime import datetime
from unittest import TestCase

from basketball_reference_web_scraper.client import regular_season_player_box_scores
from basketball_reference_web_scraper.data import Team, Outcome, OutputType
from basketball_reference_web_scraper.errors import InvalidPlayerAndSeason


class TestPlayerRegularSeasonBoxScores(TestCase):
    def test_get_2020_regular_season_box_scores_for_russell_westbrook(self):
        result = regular_season_player_box_scores(player_identifier="westbru01", season_end_year=2020)
        self.assertIsNotNone(result)
        self.assertTrue(len(result) > 0)

    def test_get_2019_regular_season_box_scores_for_russell_westbrook(self):
        result = regular_season_player_box_scores(player_identifier="westbru01", season_end_year=2019)
        self.assertIsNotNone(result)
        self.assertEqual(len(result), 73)

    def test_get_first_20202_regular_season_box_score_for_russell_westbrook(self):
        result = regular_season_player_box_scores(player_identifier="westbru01", season_end_year=2020)
        self.assertEqual(datetime.strptime("2019-10-24", "%Y-%m-%d").date(), result[0]["date"])
        self.assertEqual(Team.HOUSTON_ROCKETS, result[0]["team"])
        self.assertEqual(Outcome.LOSS, result[0]["outcome"])
        self.assertEqual(1972, result[0]["seconds_played"])
        self.assertEqual(Team.MILWAUKEE_BUCKS, result[0]["opponent"])
        self.assertEqual(7, result[0]["made_field_goals"])
        self.assertEqual(17, result[0]["attempted_field_goals"])
        self.assertEqual(3, result[0]["made_three_point_field_goals"])
        self.assertEqual(7, result[0]["attempted_three_point_field_goals"])
        self.assertEqual(7, result[0]["made_free_throws"])
        self.assertEqual(11, result[0]["attempted_free_throws"])
        self.assertEqual(4, result[0]["offensive_rebounds"])
        self.assertEqual(12, result[0]["defensive_rebounds"])
        self.assertEqual(7, result[0]["assists"])
        self.assertEqual(2, result[0]["steals"])
        self.assertEqual(1, result[0]["blocks"])
        self.assertEqual(3, result[0]["turnovers"])
        self.assertEqual(3, result[0]["personal_fouls"])
        self.assertEqual(24, result[0]["points_scored"])
        self.assertEqual(23.1, result[0]["game_score"])
        self.assertEqual(0, result[0]["plus_minus"])

    def test_get_season_box_scores_for_player_that_does_not_exist_raises_exception(self):
        self.assertRaisesRegex(
            InvalidPlayerAndSeason,
            'Player with identifier "Foo Bar" in season ending in 2020 is invalid',
            regular_season_player_box_scores,
            player_identifier='Foo Bar',
            season_end_year=2020,
        )

    def test_get_season_box_scores_for_invalid_season_raises_exception(self):
        # bbref won't actually 404 or 500 if the season is invalid, it'll
        # just take you to the player page with blank data
        self.assertRaisesRegex(
            InvalidPlayerAndSeason,
            'Player with identifier "westbru01" in season ending in 1991 is invalid',
            regular_season_player_box_scores,
            player_identifier='westbru01',
            season_end_year=1991,
        )

    def test_get_season_box_scores_removes_games_that_player_did_not_participate_in(self):
        # Jabari Brown was a DNP in his first game in the gamelog for the 2014-2015 season
        # https://www.basketball-reference.com/players/b/brownja01/gamelog/2015
        # The date for the DNP was 2015-03-10 while the first game he was active was on 2015-03-12
        # The first game that is returned should be on 2015-03-12
        results = regular_season_player_box_scores(player_identifier="brownja01", season_end_year=2015)
        self.assertIsNotNone(results)
        self.assertEqual(19, len(results))
        self.assertEqual(datetime.strptime("2015-03-12", "%Y-%m-%d").date(), results[0]["date"])

    def test_outputting_2019_regular_season_box_scores_for_avery_bradley_as_json(self):
        results = regular_season_player_box_scores(
            player_identifier="bradlav01",
            season_end_year=2019,
            output_type=OutputType.JSON,
        )
        self.assertIsNotNone(results)
        self.assertEqual(63, len(json.loads(results)))

    def test_outputting_2019_regular_season_box_scores_for_avery_bradley_as_csv(self):
        output_file_path = os.path.join(
            os.path.dirname(__file__),
            "./output/test-avery-2019.csv",
        )
        regular_season_player_box_scores(
            player_identifier="bradlav01",
            season_end_year=2019,
            output_type=OutputType.CSV,
            output_file_path=output_file_path,
        )



from unittest import TestCase

from basketball_reference_web_scraper.data import LEAGUE_ABBREVIATIONS_TO_LEAGUE, PlayerData, League
from basketball_reference_web_scraper.parser_service import ParserService
from basketball_reference_web_scraper.parsers import PlayerDataParser, ResourceLocationParser, LeagueAbbreviationParser


class TestPlayerDataParser(TestCase):
    def setUp(self):
        self.parser = PlayerDataParser(
            search_result_location_parser=ResourceLocationParser(
                resource_location_regex=ParserService.SEARCH_RESULT_RESOURCE_LOCATION_REGEX,
            ),
            league_abbreviation_parser=LeagueAbbreviationParser(abbreviations_to_league=LEAGUE_ABBREVIATIONS_TO_LEAGUE),
        )

    def test_parse_name(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=[]
            )
        )
        self.assertEqual(parsed_player["name"], "jaebaebae")

    def test_parse_resource_location(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=[]
            )
        )
        self.assertEqual(parsed_player["identifier"], "bryanko01")

    def test_parse_league_abbreviations_for_single_nba_abbreviation(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["NBA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.NATIONAL_BASKETBALL_ASSOCIATION})

    def test_parse_league_abbreviations_for_single_aba_abbreviation(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["ABA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.AMERICAN_BASKETBALL_ASSOCIATION})

    def test_parse_league_abbreviations_for_single_baa_abbreviation(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["BAA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.BASKETBALL_ASSOCIATION_OF_AMERICA})

    def test_parse_league_abbreviations_for_multiple_nba_abbreviations(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["NBA", "NBA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.NATIONAL_BASKETBALL_ASSOCIATION})

    def test_parse_league_abbreviations_for_multiple_aba_abbreviations(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["ABA", "ABA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.AMERICAN_BASKETBALL_ASSOCIATION})

    def test_parse_league_abbreviations_for_multiple_baa_abbreviations(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["BAA", "BAA"]
            )
        )
        self.assertEqual(parsed_player["leagues"], {League.BASKETBALL_ASSOCIATION_OF_AMERICA})

    def test_parse_league_abbreviations_for_multiple_nba_and_aba_and_baa_abbreviations(self):
        parsed_player = self.parser.parse(
            player=PlayerData(
                name="jaebaebae",
                resource_location='/players/b/bryanko01.html',
                league_abbreviations=["NBA", "ABA", "BAA", "NBA", "ABA", "BAA"]
            )
        )
        self.assertEqual(
            parsed_player["leagues"],
            {
                League.NATIONAL_BASKETBALL_ASSOCIATION,
                League.AMERICAN_BASKETBALL_ASSOCIATION,
                League.BASKETBALL_ASSOCIATION_OF_AMERICA
            }
        )




from datetime import datetime, timedelta
from unittest import TestCase

import pytz

from basketball_reference_web_scraper.parsers import ScheduledStartTimeParser


class TestScheduledStartTimeParser(TestCase):
    def test_correctly_parses_time_for_current_pm_formatting(self):
        parsed_start_time = ScheduledStartTimeParser().parse_start_time(
            formatted_date="Tue, Oct 17, 2017",
            formatted_time_of_day="8:01p"
        )
        expected_datetime = pytz.timezone("US/Eastern")\
            .localize(datetime(year=2017, month=10, day=17, hour=20, minute=1))\
            .astimezone(pytz.utc)

        self.assertTrue(abs(parsed_start_time - expected_datetime) < timedelta(seconds=1))

    def test_correctly_parses_time_for_current_am_formatting(self):
        parsed_start_time = ScheduledStartTimeParser().parse_start_time(
            formatted_date="Tue, Oct 17, 2017",
            formatted_time_of_day="8:01a"
        )
        expected_datetime = pytz.timezone("US/Eastern") \
            .localize(datetime(year=2017, month=10, day=17, hour=8, minute=1)) \
            .astimezone(pytz.utc)

        self.assertTrue(abs(parsed_start_time - expected_datetime) < timedelta(seconds=1))

    def test_correctly_parses_time_for_previous_pm_formatting(self):
        parsed_start_time = ScheduledStartTimeParser().parse_start_time(
            formatted_date="Tue, Oct 17, 2017",
            formatted_time_of_day="7:30 pm"
        )
        expected_datetime = pytz.timezone("US/Eastern") \
            .localize(datetime(year=2017, month=10, day=17, hour=19, minute=30)) \
            .astimezone(pytz.utc)

        self.assertTrue(abs(parsed_start_time - expected_datetime) < timedelta(seconds=1))

    def test_correctly_parses_time_for_previous_am_formatting(self):
        parsed_start_time = ScheduledStartTimeParser().parse_start_time(
            formatted_date="Tue, Oct 17, 2017",
            formatted_time_of_day="7:30 am"
        )
        expected_datetime = pytz.timezone("US/Eastern") \
            .localize(datetime(year=2017, month=10, day=17, hour=7, minute=30)) \
            .astimezone(pytz.utc)

        self.assertTrue(abs(parsed_start_time - expected_datetime) < timedelta(seconds=1))



import csv
import json

from basketball_reference_web_scraper.utilities import merge_two_dicts

# I wrote the explicit mapping of CSV values because there didn't seem to be a way of outputting the values of enums
# without doing it this way

SHARED_COLUMN_NAMES = [
    "team",
    "location",
    "opponent",
    "outcome",
    "seconds_played",
    "made_field_goals",
    "attempted_field_goals",
    "made_three_point_field_goals",
    "attempted_three_point_field_goals",
    "made_free_throws",
    "attempted_free_throws",
    "offensive_rebounds",
    "defensive_rebounds",
    "assists",
    "steals",
    "blocks",
    "turnovers",
    "personal_fouls",
    "game_score",
]

BOX_SCORE_COLUMN_NAMES = ["slug", "name"] + SHARED_COLUMN_NAMES

PLAYER_SEASON_BOX_SCORE_COLUMN_NAMES = ["date"] + SHARED_COLUMN_NAMES

SCHEDULE_COLUMN_NAMES = [
    "start_time",
    "away_team",
    "away_team_score",
    "home_team",
    "home_team_score",
]

PLAYER_SEASON_TOTALS_COLUMN_NAMES = [
    "slug",
    "name",
    "positions",
    "age",
    "team",
    "games_played",
    "games_started",
    "minutes_played",
    "made_field_goals",
    "attempted_field_goals",
    "made_three_point_field_goals",
    "attempted_three_point_field_goals",
    "made_free_throws",
    "attempted_free_throws",
    "offensive_rebounds",
    "defensive_rebounds",
    "assists",
    "steals",
    "blocks",
    "turnovers",
    "personal_fouls",
]

PLAYER_ADVANCED_SEASON_TOTALS_COLUMN_NAMES = [
    "slug",
    "name",
    "positions",
    "age",
    "team",
    "games_played",
    "minutes_played",
    "player_efficiency_rating",
    "true_shooting_percentage",
    "three_point_attempt_rate",
    "free_throw_attempt_rate",
    "offensive_rebound_percentage",
    "defensive_rebound_percentage",
    "total_rebound_percentage",
    "assist_percentage",
    "steal_percentage",
    "block_percentage",
    "turnover_percentage",
    "usage_percentage",
    "offensive_win_shares",
    "defensive_win_shares",
    "win_shares",
    "win_shares_per_48_minutes",
    "offensive_box_plus_minus",
    "defensive_box_plus_minus",
    "box_plus_minus",
    "value_over_replacement_player",
]

TEAM_BOX_SCORES_COLUMN_NAMES = [
    "team",
    "minutes_played",
    "made_field_goals",
    "attempted_field_goals",
    "made_three_point_field_goals",
    "attempted_three_point_field_goals",
    "made_free_throws",
    "attempted_free_throws",
    "offensive_rebounds",
    "defensive_rebounds",
    "assists",
    "steals",
    "blocks",
    "turnovers",
    "personal_fouls",
]

PLAY_BY_PLAY_COLUMN_NAMES = [
    "period",
    "period_type",
    "remaining_seconds_in_period",
    "relevant_team",
    "away_team",
    "home_team",
    "away_score",
    "home_score",
    "description",
]

SEARCH_RESULTS_COLUMN_NAMES = [
    "name",
    "identifier",
    "leagues",
]


class WriteOptions:
    def __init__(self, file_path=None, mode=None, custom_options=None):
        self.file_path = file_path
        self.mode = mode
        self.custom_options = custom_options

    def should_write_to_file(self):
        return self.file_path is not None and self.mode is not None

    def __eq__(self, other):
        if isinstance(other, WriteOptions):
            return self.file_path == other.file_path \
                   and self.mode == other.mode \
                   and self.custom_options == other.custom_options
        return False


class JSONWriter:
    DEFAULT_OPTIONS = {
        "sort_keys": True,
        "indent": 4,
    }

    def __init__(self, encoder):
        self.encoder = encoder

    def write(self, data, options):
        output_options = self.DEFAULT_OPTIONS \
            if options.custom_options is None \
            else merge_two_dicts(
                first=self.DEFAULT_OPTIONS,
                second=options.custom_options
            )

        if options.should_write_to_file():
            with open(options.file_path, options.mode.value, newline="", encoding="utf8") as json_file:
                return json.dump(data, json_file, cls=self.encoder, **output_options)

        return json.dumps(data, cls=self.encoder, **output_options)


class CSVWriter:
    def __init__(self, column_names, row_formatter):
        self.column_names = column_names
        self.row_formatter = row_formatter

    def format_rows(self, data):
        return [self.row_formatter.format(row_data) for row_data in data]

    def write(self, data, options):
        with open(options.file_path, options.mode.value, newline="", encoding="utf8") as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=self.column_names)
            writer.writeheader()
            writer.writerows(self.format_rows(data=data))


class SearchResultsCSVWriter(CSVWriter):
    def format_rows(self, data):
        return [
            self.row_formatter.format(row_data=row_data)
            for row_data in data["players"]
        ]


class RowFormatter:
    def __init__(self, data_field_names):
        self.data_field_names = data_field_names

    @staticmethod
    def format_field_data(field_name, field_data):
        if field_name in [
            "away_team",
            "home_team",
            "team",
            "location",
            "opponent",
            "outcome",
            "relevant_team",
            "period_type",
            "leagues",
        ]:
            if field_data is None:
                return None
            if isinstance(field_data, set):
                return "-".join(map(lambda data: data.value, list(field_data)))
            return field_data.value
        elif field_name == "positions":
            return "-".join(map(lambda position: position.value, field_data))
        else:
            return field_data

    def format(self, row_data):
        return {
            data_field_name: self.format_field_data(
                field_name=data_field_name,
                field_data=row_data[data_field_name],
            )
            for data_field_name in self.data_field_names
        }



from unittest import TestCase

from basketball_reference_web_scraper.data import PlayerData


class TestPlayerData(TestCase):
    def test_instantiation(self):
        data = PlayerData(
            name="some name",
            resource_location="some location",
            league_abbreviations=["NBA", "ABA", "NBA", "ABA"],
        )
        self.assertEqual(data.name, "some name")
        self.assertEqual(data.resource_location, "some location")
        self.assertEqual(data.league_abbreviations, {"NBA", "ABA"})



from unittest import TestCase, mock

from requests import HTTPError, codes

from basketball_reference_web_scraper.client import player_box_scores
from basketball_reference_web_scraper.errors import InvalidDate
from basketball_reference_web_scraper.http_service import HTTPService


class TestPlayerBoxScores(TestCase):
    @mock.patch.object(HTTPService, 'player_box_scores')
    def test_raises_invalid_date_for_404_response(self, mocked_player_box_scores):
        mocked_player_box_scores.side_effect = HTTPError(response=mock.Mock(status_code=codes.not_found))
        self.assertRaises(InvalidDate, player_box_scores, day=1, month=1, year=2018)

    @mock.patch.object(HTTPService, 'player_box_scores')
    def test_raises_non_404_http_error(self, mocked_player_box_scores):
        mocked_player_box_scores.side_effect = HTTPError(response=mock.Mock(status_code=codes.server_error))
        self.assertRaises(HTTPError, player_box_scores, day=1, month=1, year=2018)



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import PlayerSeasonBoxScoresRow


class TestPlayerSeasonBoxScoresRow(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_not_equal_when_row_is_compared_against_non_row(self):
        self.assertNotEqual(1, PlayerSeasonBoxScoresRow(html=MagicMock()))

    def test_not_equal_when_both_rows_but_different_html(self):
        self.assertNotEqual(
            PlayerSeasonBoxScoresRow(html=MagicMock(name="first html")),
            PlayerSeasonBoxScoresRow(html=MagicMock(name="second html")),
        )

    def test_equal_when_both_rows_and_same_html(self):
        html = MagicMock(name="shared html")
        self.assertEqual(
            PlayerSeasonBoxScoresRow(html=html),
            PlayerSeasonBoxScoresRow(html=html),
        )

    def test_is_active_is_false_when_cells_exist(self):
        cell = MagicMock()
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertFalse(PlayerSeasonBoxScoresRow(html=self.html).is_active)
        self.html.xpath.assert_called_once_with('td[@data-stat="reason"]')

    def test_is_active_is_true_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertTrue(PlayerSeasonBoxScoresRow(html=self.html).is_active)
        self.html.xpath.assert_called_once_with('td[@data-stat="reason"]')

    def test_date_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some date"))])
        self.assertEqual(PlayerSeasonBoxScoresRow(html=self.html).date, "some date")
        self.html.xpath.assert_called_once_with('td[@data-stat="date_game"]')

    def test_date_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerSeasonBoxScoresRow(html=self.html).date, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="date_game"]')

    def test_points_scored_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some points"))])
        self.assertEqual(PlayerSeasonBoxScoresRow(html=self.html).points_scored, "some points")
        self.html.xpath.assert_called_once_with('td[@data-stat="pts"]')

    def test_points_scored_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerSeasonBoxScoresRow(html=self.html).points_scored, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="pts"]')



from unittest import TestCase
from unittest.mock import patch, PropertyMock, MagicMock

from basketball_reference_web_scraper.html import PlayerSearchResult


class TestPlayerSearchResult(TestCase):
    def test_league_abbreviation_query(self):
        self.assertEqual(
            PlayerSearchResult(html=MagicMock()).league_abbreviation_query,
            './div[@class="search-item-league"]'
        )

    @patch.object(PlayerSearchResult, 'league_abbreviation_query', new_callable=PropertyMock)
    def test_league_abbreviations_are_none_when_no_matching_abbreviations(self, mocked_query):
        mocked_query.return_value = "some query"

        html = MagicMock()
        html.xpath = MagicMock(return_value=[])

        self.assertIsNone(PlayerSearchResult(html=html).league_abbreviations)
        html.xpath.assert_called_once_with("some query")

    @patch.object(PlayerSearchResult, 'league_abbreviation_query', new_callable=PropertyMock)
    def test_league_abbreviations_are_first_abbreviation_text_content_when__matching_abbreviations(self, mocked_query):
        mocked_query.return_value = "some query"
        first_abbreviation = MagicMock()
        first_abbreviation.text_content = MagicMock(return_value="first abbreviation")

        second_abbreviation = MagicMock()
        second_abbreviation.text_content = MagicMock(return_value="second abbreviation")

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_abbreviation, second_abbreviation])

        self.assertEqual(
            PlayerSearchResult(html=html).league_abbreviations,
            "first abbreviation",
        )
        html.xpath.assert_called_once_with("some query")



from unittest import TestCase, mock

from requests import codes

from basketball_reference_web_scraper.errors import InvalidDate
from basketball_reference_web_scraper.http_service import HTTPService


class TestHTTPService(TestCase):
    @mock.patch("requests.get")
    def test_player_box_scores_raises_invalid_date_for_300_response(self, mocked_get):
        response = mock.Mock(status_code=codes.multiple_choices)
        mocked_get.return_value = response
        self.assertRaisesRegex(
            InvalidDate,
            "Date with year set to 2018, month set to 1, and day set to 1 is invalid",
            HTTPService(parser=mock.MagicMock()).player_box_scores,
            day=1, month=1, year=2018)



from unittest import TestCase
from unittest.mock import MagicMock, patch, PropertyMock

from basketball_reference_web_scraper.html import PlayerIdentificationRow


class TestPlayerIdentificationRow(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_player_cell_when_cells_exist(self):
        cell = MagicMock()
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerIdentificationRow(html=self.html).player_cell, cell)
        self.html.xpath.assert_called_once_with('td[@data-stat="player"]')

    def test_player_cell_is_none_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertIsNone(PlayerIdentificationRow(html=self.html).player_cell)
        self.html.xpath.assert_called_once_with('td[@data-stat="player"]')

    @patch.object(PlayerIdentificationRow, 'player_cell', new_callable=PropertyMock)
    def test_slug_when_player_cell_is_not_none(self, mocked_player_cell):
        cell = MagicMock(get=MagicMock(return_value="some slug"))
        mocked_player_cell.return_value = cell
        self.assertEqual(PlayerIdentificationRow(html=self.html).slug, "some slug")
        cell.get.assert_called_once_with('data-append-csv')

    @patch.object(PlayerIdentificationRow, 'player_cell', new_callable=PropertyMock)
    def test_slug_when_player_cell_is_none(self, mocked_player_cell):
        mocked_player_cell.return_value = None
        self.assertEqual(PlayerIdentificationRow(html=self.html).slug, "")

    @patch.object(PlayerIdentificationRow, 'player_cell', new_callable=PropertyMock)
    def test_name_when_player_cell_is_not_none(self, mocked_player_cell):
        cell = MagicMock(text_content=MagicMock(return_value="some name"))
        mocked_player_cell.return_value = cell
        self.assertEqual(PlayerIdentificationRow(html=self.html).name, "some name")
        cell.text_content.assert_called_once_with()

    @patch.object(PlayerIdentificationRow, 'player_cell', new_callable=PropertyMock)
    def test_name_when_player_cell_is_none(self, mocked_player_cell):
        mocked_player_cell.return_value = None
        self.assertEqual(PlayerIdentificationRow(html=self.html).name, "")



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import PlayerPageTotalsTable, PlayerPageTotalsRow


class TestPlayerPageTotalsTable(TestCase):
    def test_rows_are_empty_array_when_no_results(self):
        html = MagicMock()
        html.xpath = MagicMock(return_value=[])

        self.assertEqual(
            PlayerPageTotalsTable(html=html).rows,
            []
        )
        html.xpath.assert_called_once_with('.//tbody/tr')

    def test_rows_when_results(self):
        first_row = MagicMock(name="first row html")
        second_row = MagicMock(name="second row html")

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_row, second_row])

        self.assertEqual(
            PlayerPageTotalsTable(html=html).rows,
            [
                PlayerPageTotalsRow(html=first_row),
                PlayerPageTotalsRow(html=second_row),
            ]
        )
        html.xpath.assert_called_once_with('.//tbody/tr')

    def test_different_class_is_not_equal(self):
        self.assertNotEqual(
            PlayerPageTotalsTable(html=MagicMock()),
            "jaebaebae"
        )

    def test_different_html_but_same_class_is_not_equal(self):
        self.assertNotEqual(
            PlayerPageTotalsTable(html=MagicMock()),
            PlayerPageTotalsTable(html=MagicMock())
        )

    def test_same_html_and_same_class_is_equal(self):
        html = MagicMock()
        self.assertEqual(
            PlayerPageTotalsTable(html=html),
            PlayerPageTotalsTable(html=html),
        )



from unittest import TestCase
from unittest.mock import patch, MagicMock, PropertyMock

from basketball_reference_web_scraper.html import SearchPage, PlayerSearchResult


class TestSearchPage(TestCase):
    def test_nba_aba_baa_players_content_query(self):
        self.assertEqual(
            SearchPage(html=MagicMock()).nba_aba_baa_players_content_query,
            '//div[@id="searches"]/div[@id="players"]',
        )

    @patch.object(SearchPage, 'nba_aba_baa_players_content_query', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_links_query(self, mocked_query):
        mocked_query.return_value = "some query"

        self.assertEqual(
            SearchPage(html=MagicMock()).nba_aba_baa_players_pagination_links_query,
            'some query/div[@class="search-pagination"]/a',
        )

    @patch.object(SearchPage, 'nba_aba_baa_players_content_query', new_callable=PropertyMock)
    def test_nba_aba_baa_player_search_items_query(self, mocked_query):
        mocked_query.return_value = "some query"

        self.assertEqual(
            SearchPage(html=MagicMock()).nba_aba_baa_player_search_items_query,
            'some query/div[@class="search-item"]',
        )

    @patch.object(SearchPage, 'nba_aba_baa_players_pagination_links_query', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_links(self, mocked_query):
        mocked_query.return_value = "some query"
        html = MagicMock()
        links = [MagicMock(return_value="some"), MagicMock(return_value="links")]
        html.xpath = MagicMock(return_value=links)

        self.assertEqual(
            SearchPage(html=html).nba_aba_baa_players_pagination_links,
            links,
        )
        html.xpath.asset_called_once_with("some query")

    @patch.object(SearchPage, 'nba_aba_baa_players_pagination_links', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_url_is_none_when_no_pagination_links(self, mocked_links):
        mocked_links.return_value = []
        self.assertIsNone(SearchPage(html=MagicMock()).nba_aba_baa_players_pagination_url)

    @patch.object(SearchPage, 'nba_aba_baa_players_pagination_links', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_url_is_first_link_href_attrib_when_single_link_is_not_at_end_of_results(
            self,
            mocked_links
    ):
        link = MagicMock()
        link.text_content = MagicMock(return_value="jaebaebae")
        link.attrib = MagicMock()
        link.attrib.__getitem__ = MagicMock(return_value="some text content")
        mocked_links.return_value = [link]

        self.assertEqual(
            SearchPage(html=MagicMock()).nba_aba_baa_players_pagination_url,
            "some text content",
        )
        link.attrib.__getitem__.assert_called_once_with("href")

    @patch.object(SearchPage, 'nba_aba_baa_players_pagination_links', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_url_is_none_when_single_link_is_at_end_of_results(
            self,
            mocked_links
    ):
        link = MagicMock()
        link.text_content = MagicMock(return_value="Previous 100 Results")
        mocked_links.return_value = [link]

        self.assertIsNone(SearchPage(html=MagicMock()).nba_aba_baa_players_pagination_url)
        link.text_content.assert_called_once_with()

    @patch.object(SearchPage, 'nba_aba_baa_players_pagination_links', new_callable=PropertyMock)
    def test_nba_aba_baa_players_pagination_url_is_second_link_href_attrib_when_multiple_links(
            self,
            mocked_links
    ):
        first_link = MagicMock()
        first_link.attrib = MagicMock()
        first_link.attrib.__getitem__ = MagicMock(return_value="some text content")

        second_link = MagicMock()
        second_link.attrib = MagicMock()
        second_link.attrib.__getitem__ = MagicMock(return_value="some other text content")
        mocked_links.return_value = [first_link, second_link]

        self.assertEqual(
            SearchPage(html=MagicMock()).nba_aba_baa_players_pagination_url,
            "some other text content",
        )
        second_link.attrib.__getitem__.assert_called_once_with("href")

    @patch.object(SearchPage, 'nba_aba_baa_player_search_items_query', new_callable=PropertyMock)
    def test_nba_aba_baa_players(self, mocked_query):
        mocked_query.return_value = "some query"

        first_result = MagicMock(name="first html result")
        second_result = MagicMock(name="second html result")
        third_result = MagicMock(name="third html result")

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_result, second_result, third_result])

        self.assertEqual(
            SearchPage(html=html).nba_aba_baa_players,
            [
                PlayerSearchResult(html=first_result),
                PlayerSearchResult(html=second_result),
                PlayerSearchResult(html=third_result),
            ]
        )



from unittest import TestCase

from basketball_reference_web_scraper.parser_service import ParserService
from basketball_reference_web_scraper.parsers import PeriodTimestampParser


class TestPeriodTimestampParser(TestCase):
    def setUp(self):
        self.parser = PeriodTimestampParser(timestamp_format=ParserService.PLAY_BY_PLAY_TIMESTAMP_FORMAT)

    def test_less_than_a_minute_to_seconds(self):
        self.assertEqual(32.1, self.parser.to_seconds(timestamp="0:32.1"))

    def test_more_than_a_minute_to_seconds(self):
        self.assertEqual(684.5, self.parser.to_seconds(timestamp="11:24.5"))



from unittest import TestCase

from basketball_reference_web_scraper.parsers import SecondsPlayedParser


class TestSecondsPlayedParser(TestCase):
    def setUp(self):
        self.parser = SecondsPlayedParser()

    def test_parse_seconds_played_for_empty_string(self):
        self.assertEqual(0, self.parser.parse(""))

    def test_parse_seconds_played_for_0_seconds(self):
        self.assertEqual(1, self.parser.parse("0:01"))

    def test_parse_seconds_played_for_59_seconds(self):
        self.assertEqual(59, self.parser.parse("0:59"))

    def test_parse_seconds_played_for_60_seconds(self):
        self.assertEqual(60, self.parser.parse("1:00"))

    def test_parse_seconds_played_for_61_seconds(self):
        self.assertEqual(61, self.parser.parse("1:01"))

    def test_parse_seconds_played_for_59_minutes_59_seconds(self):
        self.assertEqual(3599, self.parser.parse("59:59"))

    def test_parse_seconds_played_for_60_minutes(self):
        self.assertEqual(3600, self.parser.parse("60:00"))

    def test_parse_seconds_played_for_60_minutes_and_1_second(self):
        self.assertEqual(3601, self.parser.parse("60:01"))



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import PlayerBoxScoreRow


class TestPlayerBoxScoreRow(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_not_equal_when_row_is_compared_against_non_row(self):
        self.assertNotEqual(1, PlayerBoxScoreRow(html=MagicMock()))

    def test_not_equal_when_both_rows_but_different_html(self):
        self.assertNotEqual(
            PlayerBoxScoreRow(html=MagicMock(name="first html")),
            PlayerBoxScoreRow(html=MagicMock(name="second html")),
        )

    def test_equal_when_both_rows_and_same_html(self):
        html = MagicMock(name="shared html")
        self.assertEqual(
            PlayerBoxScoreRow(html=html),
            PlayerBoxScoreRow(html=html),
        )

    def test_team_abbreviation_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some team abbreviation"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).team_abbreviation, "some team abbreviation")
        self.html.xpath.assert_called_once_with('td[@data-stat="team_id"]')

    def test_team_abbreviation_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).team_abbreviation, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="team_id"]')

    def test_location_abbreviation_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some location abbreviation"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).location_abbreviation, "some location abbreviation")
        self.html.xpath.assert_called_once_with('td[@data-stat="game_location"]')

    def test_location_abbreviation_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).location_abbreviation, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="game_location"]')

    def test_opponent_abbreviation_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some opponent abbreviation"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).opponent_abbreviation, "some opponent abbreviation")
        self.html.xpath.assert_called_once_with('td[@data-stat="opp_id"]')

    def test_opponent_abbreviation_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).opponent_abbreviation, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="opp_id"]')

    def test_outcome_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some outcome"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).outcome, "some outcome")
        self.html.xpath.assert_called_once_with('td[@data-stat="game_result"]')

    def test_outcome_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).outcome, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="game_result"]')

    def test_plus_minus_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some plus minus"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).plus_minus, "some plus minus")
        self.html.xpath.assert_called_once_with('td[@data-stat="plus_minus"]')

    def test_plus_minus_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).plus_minus, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="plus_minus"]')

    def test_game_score_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some game score"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).game_score, "some game score")
        self.html.xpath.assert_called_once_with('td[@data-stat="game_score"]')

    def test_game_score_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(PlayerBoxScoreRow(html=self.html).game_score, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="game_score"]')




import json
import os
from unittest import TestCase

from basketball_reference_web_scraper import client
from basketball_reference_web_scraper.data import Position, Team, OutputType


class BaseTestPlayerAdvancedSeasonTotalsCSVOutput(TestCase):
    @property
    def year(self):
        raise NotImplementedError

    def setUp(self):
        self.output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/player_season_totals_{year}.csv".format(year=self.year),
        )
        self.expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_season_totals_{year}.csv".format(year=self.year),
        )

    def tearDown(self):
        os.remove(self.output_file_path)

    def assert_csv(self):
        client.players_season_totals(
            season_end_year=self.year,
            output_type=OutputType.CSV,
            output_file_path=self.output_file_path,
        )

        with open(self.output_file_path, "r", encoding="utf8") as output_file, \
                open(self.expected_output_file_path, "r", encoding="utf8") as expected_output_file:
            self.assertEqual(
                output_file.readlines(),
                expected_output_file.readlines(),
            )


class BaseTestPlayerAdvancedSeasonTotalsJSONOutput(TestCase):
    @property
    def year(self):
        raise NotImplementedError

    def setUp(self):
        self.output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/player_season_totals_{year}.json".format(year=self.year),
        )
        self.expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_season_totals_{year}.json".format(year=self.year),
        )

    def tearDown(self):
        os.remove(self.output_file_path)

    def assert_json(self):
        client.players_season_totals(
            season_end_year=self.year,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
        )

        with open(self.output_file_path, "r", encoding="utf8") as output_file, \
                open(self.expected_output_file_path, "r", encoding="utf8") as expected_output_file:
            self.assertEqual(
                json.load(output_file),
                json.load(expected_output_file),
            )


class BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput(TestCase):
    @property
    def year(self):
        raise NotImplementedError

    def setUp(self):
        self.expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_season_totals_{year}.json".format(year=self.year),
        )

    def assert_json(self):
        results = client.players_season_totals(
            season_end_year=self.year,
            output_type=OutputType.JSON,
        )

        with open(self.expected_output_file_path, "r", encoding="utf8") as expected_output_file:
            self.assertEqual(
                json.loads(results),
                json.load(expected_output_file),
            )


class Test2001PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2001

    def test_2001_csv_output(self):
        self.assert_csv()


class Test2002PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2002

    def test_2002_csv_output(self):
        self.assert_csv()


class Test2003PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2003

    def test_2003_csv_output(self):
        self.assert_csv()


class Test2004PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2004

    def test_2004_csv_output(self):
        self.assert_csv()


class Test2005PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2005

    def test_2005_csv_output(self):
        self.assert_csv()


class Test2006PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2006

    def test_2006_csv_output(self):
        self.assert_csv()


class Test2007PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2007

    def test_2007_csv_output(self):
        self.assert_csv()


class Test2008PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2008

    def test_2008_csv_output(self):
        self.assert_csv()


class Test2009PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2009

    def test_2009_csv_output(self):
        self.assert_csv()


class Test2010PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2010

    def test_2010_csv_output(self):
        self.assert_csv()


class Test2011PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2011

    def test_2011_csv_output(self):
        self.assert_csv()


class Test2012PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2012

    def test_2012_csv_output(self):
        self.assert_csv()


class Test2013PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2013

    def test_2013_csv_output(self):
        self.assert_csv()


class Test2014PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2014

    def test_2014_csv_output(self):
        self.assert_csv()


class Test2015PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2015

    def test_2015_csv_output(self):
        self.assert_csv()


class Test2016PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2016

    def test_2016_csv_output(self):
        self.assert_csv()


class Test2017PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2017

    def test_2017_csv_output(self):
        self.assert_csv()


class Test2018PlayerSeasonCSVTotals(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2018

    def test_2018_csv_output(self):
        self.assert_csv()


class Test2001PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2001

    def test_2001_json_output(self):
        self.assert_json()


class Test2002PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2002

    def test_2002_json_output(self):
        self.assert_json()


class Test2003PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2003

    def test_2003_json_output(self):
        self.assert_json()


class Test2004PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2004

    def test_2004_json_output(self):
        self.assert_json()


class Test2005PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2005

    def test_2005_json_output(self):
        self.assert_json()


class Test2006PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2006

    def test_2006_json_output(self):
        self.assert_json()


class Test2007PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2007

    def test_2007_json_output(self):
        self.assert_json()


class Test2008PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2008

    def test_2008_json_output(self):
        self.assert_json()


class Test2009PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2009

    def test_2009_json_output(self):
        self.assert_json()


class Test2010PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2010

    def test_2010_json_output(self):
        self.assert_json()


class Test2011PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2011

    def test_2011_json_output(self):
        self.assert_json()


class Test2012PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2012

    def test_2012_json_output(self):
        self.assert_json()


class Test2013PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2013

    def test_2013_json_output(self):
        self.assert_json()


class Test2014PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2014

    def test_2014_json_output(self):
        self.assert_json()


class Test2015PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2015

    def test_2015_json_output(self):
        self.assert_json()


class Test2016PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2016

    def test_2016_json_output(self):
        self.assert_json()


class Test2017PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2017

    def test_2017_json_output(self):
        self.assert_json()


class Test2018PlayerSeasonJSONTotals(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2018

    def test_2018_json_output(self):
        self.assert_json()


class Test2001PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2001

    def test_2001_json_output(self):
        self.assert_json()


class Test2002PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2002

    def test_2002_json_output(self):
        self.assert_json()


class Test2003PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2003

    def test_2003_json_output(self):
        self.assert_json()


class Test2004PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2004

    def test_2004_json_output(self):
        self.assert_json()


class Test2005PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2005

    def test_2005_json_output(self):
        self.assert_json()


class Test2006PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2006

    def test_2006_json_output(self):
        self.assert_json()


class Test2007PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2007

    def test_2007_json_output(self):
        self.assert_json()


class Test2008PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2008

    def test_2008_json_output(self):
        self.assert_json()


class Test2009PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2009

    def test_2009_json_output(self):
        self.assert_json()


class Test2010PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2010

    def test_2010_json_output(self):
        self.assert_json()


class Test2011PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2011

    def test_2011_json_output(self):
        self.assert_json()


class Test2012PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2012

    def test_2012_json_output(self):
        self.assert_json()


class Test2013PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2013

    def test_2013_json_output(self):
        self.assert_json()


class Test2014PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2014

    def test_2014_json_output(self):
        self.assert_json()


class Test2015PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2015

    def test_2015_json_output(self):
        self.assert_json()


class Test2016PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2016

    def test_2016_json_output(self):
        self.assert_json()


class Test2017PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2017

    def test_2017_json_output(self):
        self.assert_json()


class Test2018PlayerSeasonInMemoryJSONTotals(BaseTestPlayerAdvancedSeasonTotalsInMemoryJSONOutput):
    @property
    def year(self):
        return 2018

    def test_2018_json_output(self):
        self.assert_json()


class TestInMemoryPlayerSeasonTotals(TestCase):
    def test_2001_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2001)
        self.assertEqual(len(player_season_totals), 490)

    def test_first_2001_player_season_totals(self):
        player_season_totals = client.players_season_totals(season_end_year=2001)
        self.assertEqual(
            player_season_totals[0],
            {
                "slug": "abdulma02",
                "name": "Mahmoud Abdul-Rauf",
                "positions": [Position.POINT_GUARD],
                "age": 31,
                "team": Team.VANCOUVER_GRIZZLIES,
                "games_played": 41,
                "games_started": 0,
                "minutes_played": 486,
                "made_field_goals": 120,
                "attempted_field_goals": 246,
                "made_three_point_field_goals": 4,
                "attempted_three_point_field_goals": 14,
                "made_free_throws": 22,
                "attempted_free_throws": 29,
                "offensive_rebounds": 5,
                "defensive_rebounds": 20,
                "assists": 76,
                "steals": 9,
                "blocks": 1,
                "turnovers": 26,
                "personal_fouls": 50,
                "points": 266,
            }
        )

    def test_last_2001_player_season_totals(self):
        player_season_totals = client.players_season_totals(season_end_year=2001)
        self.assertEqual(
            player_season_totals[len(player_season_totals) - 1],
            {
                "slug": "zhizhwa01",
                "name": "Wang Zhizhi",
                "positions": [Position.CENTER],
                "age": 23,
                "team": Team.DALLAS_MAVERICKS,
                "games_played": 5,
                "games_started": 0,
                "minutes_played": 38,
                "made_field_goals": 8,
                "attempted_field_goals": 19,
                "made_three_point_field_goals": 0,
                "attempted_three_point_field_goals": 2,
                "made_free_throws": 8,
                "attempted_free_throws": 10,
                "offensive_rebounds": 1,
                "defensive_rebounds": 6,
                "assists": 0,
                "steals": 0,
                "blocks": 0,
                "turnovers": 1,
                "personal_fouls": 8,
                "points": 24,
            }
        )

    def test_2002_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2002)
        self.assertEqual(len(player_season_totals), 470)

    def test_2003_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2003)
        self.assertEqual(len(player_season_totals), 456)

    def test_2004_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2004)
        self.assertEqual(len(player_season_totals), 517)

    def test_2005_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2005)
        self.assertEqual(len(player_season_totals), 526)

    def test_2006_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2006)
        self.assertEqual(len(player_season_totals), 512)

    def test_2007_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2007)
        self.assertEqual(len(player_season_totals), 487)

    def test_2008_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2008)
        self.assertEqual(len(player_season_totals), 527)

    def test_2009_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2009)
        self.assertEqual(len(player_season_totals), 515)

    def test_2010_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2010)
        self.assertEqual(len(player_season_totals), 512)

    def test_2011_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2011)
        self.assertEqual(len(player_season_totals), 542)

    def test_2012_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2012)
        self.assertEqual(len(player_season_totals), 515)

    def test_2013_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2013)
        self.assertEqual(len(player_season_totals), 523)

    def test_2014_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2014)
        self.assertEqual(len(player_season_totals), 548)

    def test_2015_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2015)
        self.assertEqual(len(player_season_totals), 575)

    def test_2016_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2016)
        self.assertEqual(len(player_season_totals), 528)

    def test_2017_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2017)
        self.assertEqual(len(player_season_totals), 542)

    def test_2018_player_season_totals_length(self):
        player_season_totals = client.players_season_totals(season_end_year=2018)
        self.assertEqual(len(player_season_totals), 605)

    def test_2019_player_season_totals(self):
        player_season_totals = client.players_season_totals(season_end_year=2019)
        self.assertEqual(len(player_season_totals), 622)

    def test_avery_bradley_2019_player_season_totals(self):
        player_season_totals = client.players_season_totals(season_end_year=2019)
        clippers_avery_bradley = player_season_totals[66]

        self.assertEqual('bradlav01', clippers_avery_bradley["slug"])
        self.assertEqual("Avery Bradley", clippers_avery_bradley["name"])
        self.assertListEqual([Position.SHOOTING_GUARD], clippers_avery_bradley["positions"])
        self.assertEqual(28, clippers_avery_bradley["age"])
        self.assertEqual(Team.LOS_ANGELES_CLIPPERS, clippers_avery_bradley["team"])
        self.assertEqual(49, clippers_avery_bradley["games_played"])
        self.assertEqual(49, clippers_avery_bradley["games_started"])
        self.assertEqual(1463, clippers_avery_bradley["minutes_played"])
        self.assertEqual(161, clippers_avery_bradley["made_field_goals"])
        self.assertEqual(420, clippers_avery_bradley["attempted_field_goals"])
        self.assertEqual(58, clippers_avery_bradley["made_three_point_field_goals"])
        self.assertEqual(172, clippers_avery_bradley["attempted_three_point_field_goals"])
        self.assertEqual(20, clippers_avery_bradley["made_free_throws"])
        self.assertEqual(25, clippers_avery_bradley["attempted_free_throws"])
        self.assertEqual(35, clippers_avery_bradley["offensive_rebounds"])
        self.assertEqual(96, clippers_avery_bradley["defensive_rebounds"])
        self.assertEqual(96, clippers_avery_bradley["assists"])
        self.assertEqual(27, clippers_avery_bradley["steals"])
        self.assertEqual(16, clippers_avery_bradley["blocks"])
        self.assertEqual(61, clippers_avery_bradley["turnovers"])
        self.assertEqual(133, clippers_avery_bradley["personal_fouls"])
        self.assertEqual(400, clippers_avery_bradley["points"])



from basketball_reference_web_scraper.data import OutputType, OutputWriteOption
from basketball_reference_web_scraper.json_encoders import BasketballReferenceJSONEncoder
from basketball_reference_web_scraper.writers import JSONWriter, WriteOptions


def output(values, output_type, output_file_path, csv_writer, output_write_option=None, json_options=None):
    if output_type is None:
        return values

    write_option = OutputWriteOption.WRITE if output_write_option is None else output_write_option

    if output_type == OutputType.JSON:
        options = WriteOptions(file_path=output_file_path, mode=write_option, custom_options=json_options)
        writer = JSONWriter(encoder=BasketballReferenceJSONEncoder)
        return writer.write(data=values, options=options)

    if output_type == OutputType.CSV:
        options = WriteOptions(file_path=output_file_path, mode=write_option)
        if options.should_write_to_file():
            return csv_writer.write(data=values, options=options)
        else:
            raise ValueError("CSV output must contain a file path")

    raise ValueError("Unknown output type: {output_type}".format(output_type=output_type))



import os
from unittest import TestCase

from basketball_reference_web_scraper.client import player_box_scores
from basketball_reference_web_scraper.data import OutputType, OutputWriteOption
from basketball_reference_web_scraper.errors import InvalidDate


class TestPlayerBoxScores(TestCase):
    def test_get_box_scores(self):
        result = player_box_scores(day=1, month=1, year=2018)
        self.assertIsNotNone(result)

    def test_get_box_scores_for_day_that_does_not_exist(self):
        self.assertRaisesRegex(
            InvalidDate,
            "Date with year set to 2018, month set to 1, and day set to -1 is invalid",
            player_box_scores,
            day=-1, month=1, year=2018)

    def test_get_box_scores_from_2001(self):
        output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/2003_10_29_TOR_pbp.csv",
        )
        player_box_scores(
            day=1, month=1, year=2001,
            output_type=OutputType.CSV, output_file_path=output_file_path, output_write_option=OutputWriteOption.WRITE)

    def test_raises_invalid_date_for_nonexistent_dates(self):
        self.assertRaisesRegex(
            InvalidDate,
            "Date with year set to baz, month set to bar, and day set to foo is invalid",
            player_box_scores,
            day="foo", month="bar", year="baz")



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import ScheduleRow


class TestScheduleRow(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_instance_is_not_equal_to_different_class_instance(self):
        self.assertNotEqual(ScheduleRow(html=MagicMock()), 1)

    def test_instance_is_not_equal_to_same_class_instance_if_html_is_different(self):
        self.assertNotEqual(ScheduleRow(html=MagicMock()), ScheduleRow(html=MagicMock()))

    def test_start_date_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some start date"))])
        self.assertEqual(ScheduleRow(html=self.html).start_date, "some start date")
        self.html.xpath.assert_called_once_with('th[@data-stat="date_game"]')

    def test_start_date_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).start_date, "")
        self.html.xpath.assert_called_once_with('th[@data-stat="date_game"]')

    def test_start_time_of_day_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some start time of day"))])
        self.assertEqual(ScheduleRow(html=self.html).start_time_of_day, "some start time of day")
        self.html.xpath.assert_called_once_with('td[@data-stat="game_start_time"]')

    def test_start_time_of_day_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).start_time_of_day, "")
        self.html.xpath.assert_called_once_with('td[@data-stat="game_start_time"]')

    def test_away_team_name_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some away team name"))])
        self.assertEqual(ScheduleRow(html=self.html).away_team_name, "some away team name")
        self.html.xpath.assert_called_once_with('td[@data-stat="visitor_team_name"]')

    def test_away_team_name_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).away_team_name, "")
        self.html.xpath.assert_called_once_with('td[@data-stat="visitor_team_name"]')

    def test_home_team_name_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some home team name"))])
        self.assertEqual(ScheduleRow(html=self.html).home_team_name, "some home team name")
        self.html.xpath.assert_called_once_with('td[@data-stat="home_team_name"]')

    def test_home_team_name_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).home_team_name, "")
        self.html.xpath.assert_called_once_with('td[@data-stat="home_team_name"]')

    def test_away_team_score_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some away team score"))])
        self.assertEqual(ScheduleRow(html=self.html).away_team_score, "some away team score")
        self.html.xpath.assert_called_once_with('td[@data-stat="visitor_pts"]')

    def test_away_team_score_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).away_team_score, "")
        self.html.xpath.assert_called_once_with('td[@data-stat="visitor_pts"]')

    def test_home_team_score_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[MagicMock(text_content=MagicMock(return_value="some home team score"))])
        self.assertEqual(ScheduleRow(html=self.html).home_team_score, "some home team score")
        self.html.xpath.assert_called_once_with('td[@data-stat="home_pts"]')

    def test_home_team_score_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(ScheduleRow(html=self.html).home_team_score, "")
        self.html.xpath.assert_called_once_with('td[@data-stat="home_pts"]')



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import BasicBoxScoreRow


class TestBasicBoxScoreRow(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_playing_time_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some playing time"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).playing_time, "some playing time")
        self.html.xpath.assert_called_once_with('td[@data-stat="mp"]')

    def test_playing_time_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).playing_time, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="mp"]')

    def test_minutes_played_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some minutes played"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).minutes_played, "some minutes played")
        self.html.xpath.assert_called_once_with('td[@data-stat="mp"]')

    def test_minutes_played_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).minutes_played, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="mp"]')

    def test_made_field_goals_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some made field goals"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).made_field_goals, "some made field goals")
        self.html.xpath.assert_called_once_with('td[@data-stat="fg"]')

    def test_made_field_goals_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).made_field_goals, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="fg"]')

    def test_attempted_field_goals_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some attempted field goals"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).attempted_field_goals, "some attempted field goals")
        self.html.xpath.assert_called_once_with('td[@data-stat="fga"]')

    def test_attempted_field_goals_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).attempted_field_goals, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="fga"]')

    def test_made_three_point_field_goals_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some made three point field goals"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(
            BasicBoxScoreRow(html=self.html).made_three_point_field_goals,
            "some made three point field goals",
        )
        self.html.xpath.assert_called_once_with('td[@data-stat="fg3"]')

    def test_made_three_point_field_goals_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).made_three_point_field_goals, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="fg3"]')

    def test_attempted_three_point_field_goals_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some attempted three point field goals"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(
            BasicBoxScoreRow(html=self.html).attempted_three_point_field_goals,
            "some attempted three point field goals",
        )
        self.html.xpath.assert_called_once_with('td[@data-stat="fg3a"]')

    def test_attempted_three_point_field_goals_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).attempted_three_point_field_goals, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="fg3a"]')

    def test_made_free_throws_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some made free throws"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).made_free_throws, "some made free throws")
        self.html.xpath.assert_called_once_with('td[@data-stat="ft"]')

    def test_made_free_throws_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).made_free_throws, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="ft"]')

    def test_attempted_free_throws_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some attempted free throws"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).attempted_free_throws, "some attempted free throws")
        self.html.xpath.assert_called_once_with('td[@data-stat="fta"]')

    def test_attempted_free_throws_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).attempted_free_throws, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="fta"]')

    def test_offensive_rebounds_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some offensive rebounds"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).offensive_rebounds, "some offensive rebounds")
        self.html.xpath.assert_called_once_with('td[@data-stat="orb"]')

    def test_offensive_rebounds_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).offensive_rebounds, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="orb"]')

    def test_defensive_rebounds_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some defensive rebounds"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).defensive_rebounds, "some defensive rebounds")
        self.html.xpath.assert_called_once_with('td[@data-stat="drb"]')

    def test_defensive_rebounds_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).defensive_rebounds, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="drb"]')

    def test_assists_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some assists"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).assists, "some assists")
        self.html.xpath.assert_called_once_with('td[@data-stat="ast"]')

    def test_assists_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).assists, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="ast"]')

    def test_steals(self):
        cell = MagicMock(text_content=MagicMock(return_value="some steals"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).steals, "some steals")
        self.html.xpath.assert_called_once_with('td[@data-stat="stl"]')

    def test_steals_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).steals, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="stl"]')

    def test_blocks_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some blocks"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).blocks, "some blocks")
        self.html.xpath.assert_called_once_with('td[@data-stat="blk"]')

    def test_blocks_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).blocks, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="blk"]')

    def test_turnovers_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some turnovers"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).turnovers, "some turnovers")
        self.html.xpath.assert_called_once_with('td[@data-stat="tov"]')

    def test_turnovers_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).turnovers, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="tov"]')

    def test_personal_fouls_when_cells_exist(self):
        cell = MagicMock(text_content=MagicMock(return_value="some personal fouls"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).personal_fouls, "some personal fouls")
        self.html.xpath.assert_called_once_with('td[@data-stat="pf"]')

    def test_personal_fouls_is_empty_string_when_cells_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).personal_fouls, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="pf"]')

    def test_points(self):
        cell = MagicMock(text_content=MagicMock(return_value="some points"))
        self.html.xpath = MagicMock(return_value=[cell])
        self.assertEqual(BasicBoxScoreRow(html=self.html).points, "some points")
        self.html.xpath.assert_called_once_with('td[@data-stat="pts"]')

    def test_points_is_empty_string_when_cells_do_not_exist(self):
        self.html.xpath = MagicMock(return_value=[])
        self.assertEqual(BasicBoxScoreRow(html=self.html).points, '')
        self.html.xpath.assert_called_once_with('td[@data-stat="pts"]')



from unittest import TestCase
from unittest.mock import patch, MagicMock

import requests
from requests.exceptions import HTTPError

import basketball_reference_web_scraper.client as client
from basketball_reference_web_scraper.errors import InvalidDate
from basketball_reference_web_scraper.http_service import HTTPService


class TestTeamBoxScores(TestCase):
    @patch.object(HTTPService, "team_box_scores")
    def test_invalid_date_error_raised_for_unknown_date(self, mocked_http_team_box_scores):
        mocked_http_team_box_scores.side_effect = HTTPError(
            response=MagicMock(status_code=requests.codes.not_found)
        )
        self.assertRaisesRegex(
            InvalidDate,
            "Date with year set to jae, month set to bae, and day set to bae is invalid",
            client.team_box_scores,
            day="bae",
            month="bae",
            year="jae"
        )



from unittest import TestCase, mock

from basketball_reference_web_scraper.data import OutputWriteOption, OutputType
from basketball_reference_web_scraper.json_encoders import BasketballReferenceJSONEncoder
from basketball_reference_web_scraper.output import output
from basketball_reference_web_scraper.writers import WriteOptions


class TestOutput(TestCase):
    def setUp(self):
        self.values = ["some values"]
        self.output_file_path = "some output file path"
        self.csv_writer = mock.Mock(write=mock.Mock())

    def test_return_values_when_output_type_is_none(self):
        self.assertEqual(
            self.values,
            output(
                values=self.values,
                output_type=None,
                output_file_path=self.output_file_path,
                csv_writer=self.csv_writer,
            ),
        )

    def test_raise_error_when_output_type_is_not_json_or_csv(self):
        self.assertRaisesRegex(
            ValueError,
            "Unknown output type: some other output type",
            output,
            values=self.values,
            output_type="some other output type",
            output_file_path=None,
            csv_writer=None,
        )

    @mock.patch("basketball_reference_web_scraper.output.JSONWriter")
    def test_output_json_when_output_write_option_is_none_and_no_custom_options(
            self,
            mock_json_writer,
    ):
        mock_json_writer_instance = mock.Mock(write=mock.Mock())
        mock_json_writer.return_value = mock_json_writer_instance

        output(
            values=self.values,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
            csv_writer=None,
        )

        mock_json_writer.assert_called_once_with(encoder=BasketballReferenceJSONEncoder)
        mock_json_writer_instance.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.WRITE,
                custom_options=None,
            )
        )

    @mock.patch("basketball_reference_web_scraper.output.JSONWriter")
    def test_output_json_when_output_write_option_is_append_and_no_custom_options(
            self,
            mock_json_writer,
    ):
        mock_json_writer_instance = mock.Mock(write=mock.Mock())
        mock_json_writer.return_value = mock_json_writer_instance

        output(
            values=self.values,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
            csv_writer=None,
            output_write_option=OutputWriteOption.APPEND,
        )

        mock_json_writer.assert_called_once_with(encoder=BasketballReferenceJSONEncoder)
        mock_json_writer_instance.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.APPEND,
                custom_options=None,
            )
        )

    @mock.patch("basketball_reference_web_scraper.output.JSONWriter")
    def test_output_json_when_output_write_option_is_none_and_custom_options(
            self,
            mock_json_writer,
    ):
        mock_json_writer_instance = mock.Mock(write=mock.Mock())
        mock_json_writer.return_value = mock_json_writer_instance

        output(
            values=self.values,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
            csv_writer=None,
            json_options={
                "jae": "baebae",
                "bae": "jadley",
            },
        )

        mock_json_writer.assert_called_once_with(encoder=BasketballReferenceJSONEncoder)
        mock_json_writer_instance.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.WRITE,
                custom_options={
                    "jae": "baebae",
                    "bae": "jadley",
                },
            )
        )

    @mock.patch("basketball_reference_web_scraper.output.JSONWriter")
    def test_output_json_when_output_write_option_is_append_and_custom_options(
            self,
            mock_json_writer,
    ):
        mock_json_writer_instance = mock.Mock(write=mock.Mock())
        mock_json_writer.return_value = mock_json_writer_instance

        output(
            values=self.values,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
            csv_writer=None,
            output_write_option=OutputWriteOption.APPEND,
            json_options={
                "jae": "baebae",
                "bae": "jadley",
            },
        )

        mock_json_writer.assert_called_once_with(encoder=BasketballReferenceJSONEncoder)
        mock_json_writer_instance.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.APPEND,
                custom_options={
                    "jae": "baebae",
                    "bae": "jadley",
                },
            )
        )

    def test_output_csv_when_output_write_option_is_none_and_no_custom_options(self):
        csv_writer = mock.Mock(name="csv_writer", write=mock.Mock())
        output(
            values=self.values,
            output_type=OutputType.CSV,
            output_file_path=self.output_file_path,
            csv_writer=csv_writer,
        )

        csv_writer.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.WRITE,
            ),
        )

    def test_output_csv_when_output_write_option_is_append_and_no_custom_options(self):
        csv_writer = mock.Mock(name="csv_writer", write=mock.Mock())
        output(
            values=self.values,
            output_type=OutputType.CSV,
            output_file_path=self.output_file_path,
            csv_writer=csv_writer,
            output_write_option=OutputWriteOption.APPEND,
        )

        csv_writer.write.assert_called_once_with(
            data=self.values,
            options=WriteOptions(
                file_path=self.output_file_path,
                mode=OutputWriteOption.APPEND,
            ),
        )

    @mock.patch.object(WriteOptions, 'should_write_to_file')
    def test_raise_error_when_outputting_csv_but_unable_to_write_to_file(self, mock_should_write_to_file):
        mock_should_write_to_file.return_value = False
        csv_writer = mock.Mock(name="csv_writer", write=mock.Mock())
        self.assertRaisesRegex(
            ValueError,
            "CSV output must contain a file path",
            output,
            values=self.values,
            output_type=OutputType.CSV,
            output_file_path=self.output_file_path,
            csv_writer=csv_writer,
        )



from unittest import TestCase
from unittest.mock import MagicMock, PropertyMock
from unittest.mock import patch

from basketball_reference_web_scraper.html import PlayerAdvancedSeasonTotalsRow, PlayerAdvancedSeasonTotalsTable


class TestPlayerAdvancedSeasonTotalsTable(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_rows_query_after_stripping_whitespace(self):
        self.assertEqual(
            """
            //table[@id="advanced_stats"]
            /tbody
            /tr[
                contains(@class, "full_table") or 
                contains(@class, "italic_text partial_table") 
                and not(contains(@class, "rowSum"))
            ]
            """.strip(),
            PlayerAdvancedSeasonTotalsTable(html=self.html).rows_query.strip(),
        )

    @patch.object(PlayerAdvancedSeasonTotalsRow, 'is_combined_totals', new_callable=PropertyMock, return_value=False)
    def test_returns_all_rows_when_rows_are_not_combined_totals_rows(self, _):
        first_html_row = MagicMock()
        html_rows = [first_html_row]
        self.html.xpath = MagicMock(return_value=html_rows)

        rows = PlayerAdvancedSeasonTotalsTable(self.html).get_rows()
        self.assertTrue(len(html_rows) == len(rows))

    @patch.object(PlayerAdvancedSeasonTotalsRow, 'is_combined_totals', new_callable=PropertyMock, return_value=True)
    def test_returns_no_rows_when_all_rows_are_combined_totals_rows(self, _):
        first_html_row = MagicMock()
        html_rows = [first_html_row]
        self.html.xpath = MagicMock(return_value=html_rows)

        rows = PlayerAdvancedSeasonTotalsTable(self.html).get_rows()
        self.assertTrue(0 == len(rows))




from unittest import TestCase

import requests
from lxml import html

from basketball_reference_web_scraper.data import TEAM_ABBREVIATIONS_TO_TEAM, TeamTotal
from basketball_reference_web_scraper.data import Team, Outcome
from basketball_reference_web_scraper.html import BoxScoresPage
from basketball_reference_web_scraper.parsers import TeamAbbreviationParser, \
    TeamTotalsParser


class TestParseTeams(TestCase):
    def setUp(self):
        self.atlanta_box_score_2017_01_01 = requests.get(
            'https://www.basketball-reference.com/boxscores/201701010ATL.html'
        ).text
        self.team_abbreviation_parser = TeamAbbreviationParser(
            abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM,
        )
        self.page = BoxScoresPage(html.fromstring(self.atlanta_box_score_2017_01_01))
        self.combined_team_totals = [
            TeamTotal(team_abbreviation=table.team_abbreviation, totals=table.team_totals)
            for table in self.page.basic_statistics_tables
        ]
        self.parser = TeamTotalsParser(team_abbreviation_parser=self.team_abbreviation_parser)
        self.team_totals = self.parser.parse(
            first_team_totals=self.combined_team_totals[0],
            second_team_totals=self.combined_team_totals[1],
        )

    def test_parse_two_team_totals(self):
        self.assertEqual(len(self.team_totals), 2)

    def test_parse_san_antonio_team_totals(self):
        sas_team_totals = self.team_totals[0]
        self.assertEqual(sas_team_totals["team"], Team.SAN_ANTONIO_SPURS)
        self.assertEqual(sas_team_totals["outcome"], Outcome.LOSS)
        self.assertEqual(sas_team_totals["minutes_played"], 265)
        self.assertEqual(sas_team_totals["made_field_goals"], 42)
        self.assertEqual(sas_team_totals["attempted_field_goals"], 90)
        self.assertEqual(sas_team_totals["made_three_point_field_goals"], 9)
        self.assertEqual(sas_team_totals["attempted_three_point_field_goals"], 27)
        self.assertEqual(sas_team_totals["made_free_throws"], 19)
        self.assertEqual(sas_team_totals["attempted_free_throws"], 22)
        self.assertEqual(sas_team_totals["offensive_rebounds"], 9)
        self.assertEqual(sas_team_totals["defensive_rebounds"], 38)
        self.assertEqual(sas_team_totals["assists"], 27)
        self.assertEqual(sas_team_totals["steals"], 5)
        self.assertEqual(sas_team_totals["blocks"], 6)
        self.assertEqual(sas_team_totals["turnovers"], 12)
        self.assertEqual(sas_team_totals["personal_fouls"], 21)
        self.assertEqual(sas_team_totals["points"], 112)

    def test_parse_atlanta_team_totals(self):
        atl_team_totals = self.team_totals[1]
        self.assertEqual(atl_team_totals["team"], Team.ATLANTA_HAWKS)
        self.assertEqual(atl_team_totals["outcome"], Outcome.WIN)
        self.assertEqual(atl_team_totals["minutes_played"], 265)
        self.assertEqual(atl_team_totals["made_field_goals"], 42)
        self.assertEqual(atl_team_totals["attempted_field_goals"], 92)
        self.assertEqual(atl_team_totals["made_three_point_field_goals"], 14)
        self.assertEqual(atl_team_totals["attempted_three_point_field_goals"], 28)
        self.assertEqual(atl_team_totals["made_free_throws"], 16)
        self.assertEqual(atl_team_totals["attempted_free_throws"], 27)
        self.assertEqual(atl_team_totals["offensive_rebounds"], 11)
        self.assertEqual(atl_team_totals["defensive_rebounds"], 35)
        self.assertEqual(atl_team_totals["assists"], 25)
        self.assertEqual(atl_team_totals["steals"], 6)
        self.assertEqual(atl_team_totals["blocks"], 6)
        self.assertEqual(atl_team_totals["turnovers"], 11)
        self.assertEqual(atl_team_totals["personal_fouls"], 21)
        self.assertEqual(atl_team_totals["points"], 114)



from unittest import TestCase
from unittest.mock import MagicMock, patch, PropertyMock

from basketball_reference_web_scraper.html import SchedulePage, ScheduleRow


class TestSchedulePage(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_other_months_schedule_links_query(self):
        self.assertEqual(
            SchedulePage(html=self.html).other_months_schedule_links_query,
            '//div[@id="content"]/div[@class="filter"]/div[not(contains(@class, "current"))]/a'
        )

    def test_rows_query(self):
        self.assertEqual(
            SchedulePage(html=self.html).rows_query,
            '//table[@id="schedule"]//tbody/tr'
        )

    @patch.object(SchedulePage, 'other_months_schedule_links_query', new_callable=PropertyMock)
    def test_other_months_schedule_urls(self, mocked_other_months_schedule_links_query):
        query = "some query"
        mocked_other_months_schedule_links_query.return_value = query
        link_href = "some link href"
        link = MagicMock()
        link.attrib = MagicMock()
        link.attrib.__getitem__ = MagicMock(return_value=link_href)
        links = [link]
        self.html.xpath = MagicMock(return_value=links)

        self.assertEqual(
            SchedulePage(html=self.html).other_months_schedule_urls,
            [link_href]
        )
        self.html.xpath.assert_called_once_with(query)
        link.attrib.__getitem__.assert_called_once_with('href')

    @patch.object(SchedulePage, 'rows_query', new_callable=PropertyMock)
    def test_no_rows_are_returned_when_all_rows_have_playoffs_content(self, mocked_rows_query):
        query = "some query"
        mocked_rows_query.return_value = query
        playoff_row = MagicMock()
        playoff_row.text_content = MagicMock(return_value="Playoffs")
        rows = [playoff_row]
        self.html.xpath = MagicMock(return_value=rows)

        self.assertEqual(
            SchedulePage(html=self.html).rows,
            []
        )
        self.html.xpath.assert_called_once_with(query)
        playoff_row.text_content.assert_called_once_with()

    @patch.object(SchedulePage, 'rows_query', new_callable=PropertyMock)
    def test_all_rows_are_returned_when_all_rows_have_playoffs_content(self, mocked_rows_query):
        query = "some query"
        mocked_rows_query.return_value = query
        non_playoff_row = MagicMock()
        non_playoff_row.text_content = MagicMock(return_value="jaebaebae")
        rows = [non_playoff_row]
        self.html.xpath = MagicMock(return_value=rows)

        self.assertEqual(
            SchedulePage(html=self.html).rows,
            [
                ScheduleRow(html=non_playoff_row)
            ]
        )



from datetime import datetime, date
from enum import Enum
from json import JSONEncoder


class BasketballReferenceJSONEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime) or isinstance(obj, date):
            return obj.isoformat()

        if isinstance(obj, Enum):
            return obj.value

        if isinstance(obj, set):
            return list(obj)

        return JSONEncoder.default(self, obj)




from unittest import TestCase, mock

from basketball_reference_web_scraper.data import OutputWriteOption
from basketball_reference_web_scraper.writers import JSONWriter


class TestJSONWriter(TestCase):
    def setUp(self):
        self.mock_encoder = mock.Mock()
        self.mock_data = ["some data"]
        self.writer = JSONWriter(encoder=self.mock_encoder)

    def test_default_options(self):
        self.assertEqual({"sort_keys": True, "indent": 4}, JSONWriter.DEFAULT_OPTIONS)

    @mock.patch("basketball_reference_web_scraper.writers.json.dumps")
    def test_writing_to_memory_with_default_options(self, json_dumps):
        options = mock.Mock(
            custom_options=None,
            should_write_to_file=mock.Mock(return_value=False),
        )

        self.writer.write(data=self.mock_data, options=options)
        options.should_write_to_file.assert_called_once_with()
        json_dumps.assert_called_once_with(self.mock_data, cls=self.mock_encoder, sort_keys=True, indent=4)

    @mock.patch("basketball_reference_web_scraper.writers.json.dumps")
    def test_writing_to_memory_with_custom_options(self, json_dumps):
        options = mock.Mock(
            custom_options={
                "jae": "baebae",
                "bae": "jadley",
            },
            should_write_to_file=mock.Mock(return_value=False),
        )

        self.writer.write(data=self.mock_data, options=options)
        options.should_write_to_file.assert_called_once_with()
        json_dumps.assert_called_once_with(
            self.mock_data,
            cls=self.mock_encoder,
            sort_keys=True,
            indent=4,
            jae="baebae",
            bae="jadley",
        )

    @mock.patch("basketball_reference_web_scraper.writers.json.dump")
    def test_writing_to_file_with_default_options(self, json_dump):
        file_path = "some file path"
        with mock.patch("builtins.open", mock.mock_open()) as mock_file:
            options = mock.Mock(
                file_path=file_path,
                mode=OutputWriteOption.WRITE,
                custom_options=None,
                should_write_to_file=mock.Mock(return_value=True),
            )

            self.writer.write(data=self.mock_data, options=options)
            options.should_write_to_file.assert_called_once_with()
            mock_file.assert_called_once_with(file_path, OutputWriteOption.WRITE.value, newline="", encoding="utf8")
            json_dump.assert_called_once_with(
                self.mock_data,
                mock_file(),
                cls=self.mock_encoder,
                sort_keys=True,
                indent=4,
            )

    @mock.patch("basketball_reference_web_scraper.writers.json.dump")
    def test_writing_to_file_with_custom_options(self, json_dump):
        file_path = "some file path"
        with mock.patch("builtins.open", mock.mock_open()) as mock_file:
            options = mock.Mock(
                file_path=file_path,
                mode=OutputWriteOption.WRITE,
                custom_options={
                    "jae": "baebae",
                    "bae": "jadley",
                },
                should_write_to_file=mock.Mock(return_value=True),
            )

            self.writer.write(data=self.mock_data, options=options)
            options.should_write_to_file.assert_called_once_with()
            mock_file.assert_called_once_with(file_path, OutputWriteOption.WRITE.value, newline="", encoding="utf8")
            json_dump.assert_called_once_with(
                self.mock_data,
                mock_file(),
                cls=self.mock_encoder,
                sort_keys=True,
                indent=4,
                jae="baebae",
                bae="jadley",
            )



import json
import os
from datetime import date
from unittest import TestCase

from basketball_reference_web_scraper.client import players_advanced_season_totals
from basketball_reference_web_scraper.data import OutputType, Team, Position
from basketball_reference_web_scraper.errors import InvalidSeason


class BaseTestPlayerAdvancedSeasonTotalsCSVOutput(TestCase):
    @property
    def year(self):
        raise NotImplementedError

    @property
    def include_combined_values(self):
        raise NotImplementedError

    def setUp(self):
        self.output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/player_advanced_season_totals_{year}.csv".format(year=self.year),
        )
        self.expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_advanced_season_totals_{year}.csv".format(year=self.year),
        )

    def tearDown(self):
        os.remove(self.output_file_path)

    def assert_player_advanced_season_totals_csv(self):
        players_advanced_season_totals(
            season_end_year=self.year,
            output_type=OutputType.CSV,
            output_file_path=self.output_file_path,
            include_combined_values=self.include_combined_values,
        )

        with open(self.output_file_path, "r", encoding="utf8") as output_file, \
                open(self.expected_output_file_path, "r", encoding="utf8") as expected_output_file:
            self.assertEqual(
                output_file.readlines(),
                expected_output_file.readlines(),
            )


class BaseTestPlayerAdvancedSeasonTotalsJSONOutput(TestCase):
    @property
    def year(self):
        raise NotImplementedError

    @property
    def include_combined_values(self):
        raise NotImplementedError

    def setUp(self):
        self.output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/player_advanced_season_totals_{year}.json".format(year=self.year),
        )
        self.expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_advanced_season_totals_{year}.json".format(year=self.year),
        )

    def tearDown(self):
        os.remove(self.output_file_path)

    def assert_player_advanced_season_totals_json(self):
        players_advanced_season_totals(
            season_end_year=self.year,
            output_type=OutputType.JSON,
            output_file_path=self.output_file_path,
            include_combined_values=self.include_combined_values,
        )

        with open(self.output_file_path, "r", encoding="utf8") as output_file, \
                open(self.expected_output_file_path, "r", encoding="utf8") as expected_output_file:
            self.assertEqual(
                json.load(output_file),
                json.load(expected_output_file),
            )


class Test2018PlayerAdvancedSeasonTotalsCSVOutput(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2018

    @property
    def include_combined_values(self):
        return False

    def test_players_advanced_season_totals_csv(self):
        self.assert_player_advanced_season_totals_csv()


class Test2017PlayerAdvancedSeasonTotalsCSVOutput(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2017

    @property
    def include_combined_values(self):
        return True

    def test_players_advanced_season_totals_csv(self):
        self.assert_player_advanced_season_totals_csv()


class Test2016PlayerAdvancedSeasonTotalsCSVOutput(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2016

    @property
    def include_combined_values(self):
        return True

    def test_players_advanced_season_totals_csv(self):
        self.assert_player_advanced_season_totals_csv()


class Test2001PlayerAdvancedSeasonTotalsCSVOutput(BaseTestPlayerAdvancedSeasonTotalsCSVOutput):
    @property
    def year(self):
        return 2001

    @property
    def include_combined_values(self):
        return False

    def test_players_advanced_season_totals_csv(self):
        self.assert_player_advanced_season_totals_csv()


class Test2018PlayerAdvancedSeasonTotalsJSONOutput(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2018

    @property
    def include_combined_values(self):
        return False

    def test_players_advanced_season_totals_json(self):
        self.assert_player_advanced_season_totals_json()


class Test2017PlayerAdvancedSeasonTotalsJSONOutput(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2017

    @property
    def include_combined_values(self):
        return True

    def test_players_advanced_season_totals_json(self):
        self.assert_player_advanced_season_totals_json()


class Test2016PlayerAdvancedSeasonTotalsJSONOutput(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2016

    @property
    def include_combined_values(self):
        return True

    def test_players_advanced_season_totals_json(self):
        self.assert_player_advanced_season_totals_json()


class Test2001PlayerAdvancedSeasonTotalsJSONOutput(BaseTestPlayerAdvancedSeasonTotalsJSONOutput):
    @property
    def year(self):
        return 2001

    @property
    def include_combined_values(self):
        return False

    def test_players_advanced_season_totals_json(self):
        self.assert_player_advanced_season_totals_json()


class TestPlayerAdvancedSeasonTotalsInMemoryOutput(TestCase):
    def test_future_season_raises_invalid_season(self):
        current_year = date.today().year
        future_year = current_year + 10
        expected_message = "Season end year of {future_year} is invalid".format(future_year=future_year)
        self.assertRaisesRegex(InvalidSeason, expected_message, players_advanced_season_totals, season_end_year=future_year)

    def test_2018_players_advanced_season_totals_length(self):
        result = players_advanced_season_totals(season_end_year=2018)
        self.assertEqual(len(result), 605)

    def test_first_2018_players_advanced_season_totals_row(self):
        result = players_advanced_season_totals(season_end_year=2018)
        self.assertEqual(
            result[0],
            {
                "age": 24,
                "assist_percentage": 3.4,
                "block_percentage": 0.6,
                "box_plus_minus": -1.5,
                "defensive_box_plus_minus": 0.4,
                "defensive_rebound_percentage": 8.9,
                "defensive_win_shares": 1.0,
                "free_throw_attempt_rate": 0.158,
                "games_played": 75,
                "is_combined_totals": False,
                "minutes_played": 1134,
                "name": "\u00c1lex Abrines",
                "offensive_box_plus_minus": -1.9,
                "offensive_rebound_percentage": 2.5,
                "offensive_win_shares": 1.3,
                "player_efficiency_rating": 9.0,
                "positions": [
                    Position.SHOOTING_GUARD,
                ],
                "slug": "abrinal01",
                "steal_percentage": 1.7,
                "team": Team.OKLAHOMA_CITY_THUNDER,
                "three_point_attempt_rate": 0.759,
                "total_rebound_percentage": 5.6,
                "true_shooting_percentage": 0.567,
                "turnover_percentage": 7.4,
                "usage_percentage": 12.7,
                "value_over_replacement_player": 0.1,
                "win_shares": 2.2,
                "win_shares_per_48_minutes": 0.094
            },
        )

    def test_last_2018_players_advanced_season_totals_row(self):
        result = players_advanced_season_totals(season_end_year=2018)
        self.assertEqual(
            result[604],
            {
                "age": 20,
                "assist_percentage": 8.8,
                "block_percentage": 3.0,
                "box_plus_minus": -2.0,
                "defensive_box_plus_minus": -0.3,
                "defensive_rebound_percentage": 20.1,
                "defensive_win_shares": 0.5,
                "free_throw_attempt_rate": 0.418,
                "games_played": 43,
                "is_combined_totals": False,
                "minutes_played": 410,
                "name": "Ivica Zubac",
                "offensive_box_plus_minus": -1.8,
                "offensive_rebound_percentage": 11.8,
                "offensive_win_shares": 0.5,
                "player_efficiency_rating": 15.3,
                "positions": [
                    Position.CENTER
                ],
                "slug": "zubaciv01",
                "steal_percentage": 0.9,
                "team": Team.LOS_ANGELES_LAKERS,
                "three_point_attempt_rate": 0.008,
                "total_rebound_percentage": 16.0,
                "true_shooting_percentage": 0.557,
                "turnover_percentage": 15.3,
                "usage_percentage": 17.6,
                "value_over_replacement_player": 0.0,
                "win_shares": 1.0,
                "win_shares_per_48_minutes": 0.118
            }
        )

    def test_players_advanced_season_totals_json(self):
        expected_output_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/player_advanced_season_totals_2018.json",
        )
        result = players_advanced_season_totals(season_end_year=2018, output_type=OutputType.JSON)
        with open(expected_output_file_path, "r", encoding="utf8") as expected_output:
            self.assertEqual(
                json.loads(result),
                json.load(expected_output),
            )



import requests

from basketball_reference_web_scraper.errors import InvalidSeason, InvalidDate, InvalidPlayerAndSeason
from basketball_reference_web_scraper.http_service import HTTPService
from basketball_reference_web_scraper.output import output
from basketball_reference_web_scraper.parser_service import ParserService
from basketball_reference_web_scraper.writers import CSVWriter, RowFormatter, \
    BOX_SCORE_COLUMN_NAMES, SCHEDULE_COLUMN_NAMES, PLAYER_SEASON_TOTALS_COLUMN_NAMES, \
    PLAYER_ADVANCED_SEASON_TOTALS_COLUMN_NAMES, TEAM_BOX_SCORES_COLUMN_NAMES, PLAY_BY_PLAY_COLUMN_NAMES, \
    PLAYER_SEASON_BOX_SCORE_COLUMN_NAMES, SearchResultsCSVWriter, SEARCH_RESULTS_COLUMN_NAMES


def player_box_scores(day, month, year, output_type=None, output_file_path=None, output_write_option=None,
                      json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.player_box_scores(day=day, month=month, year=year)
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidDate(day=day, month=month, year=year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=BOX_SCORE_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=BOX_SCORE_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def regular_season_player_box_scores(player_identifier, season_end_year, output_type=None, output_file_path=None,
                                     output_write_option=None, json_options=None):

    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.regular_season_player_box_scores(
            player_identifier=player_identifier,
            season_end_year=season_end_year,
        )
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.internal_server_error \
                or http_error.response.status_code == requests.codes.not_found:
            raise InvalidPlayerAndSeason(player_identifier=player_identifier, season_end_year=season_end_year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=PLAYER_SEASON_BOX_SCORE_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=PLAYER_SEASON_BOX_SCORE_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def season_schedule(season_end_year, output_type=None, output_file_path=None, output_write_option=None,
                    json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.season_schedule(season_end_year=season_end_year)
    except requests.exceptions.HTTPError as http_error:
        # https://github.com/requests/requests/blob/master/requests/status_codes.py#L58
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidSeason(season_end_year=season_end_year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=SCHEDULE_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=SCHEDULE_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def players_season_totals(season_end_year, output_type=None, output_file_path=None, output_write_option=None,
                          json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.players_season_totals(season_end_year=season_end_year)
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidSeason(season_end_year=season_end_year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=PLAYER_SEASON_TOTALS_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=PLAYER_SEASON_TOTALS_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def players_advanced_season_totals(season_end_year, include_combined_values=False, output_type=None,
                                   output_file_path=None, output_write_option=None, json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.players_advanced_season_totals(
            season_end_year,
            include_combined_values=include_combined_values
        )
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidSeason(season_end_year=season_end_year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=PLAYER_ADVANCED_SEASON_TOTALS_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=PLAYER_ADVANCED_SEASON_TOTALS_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def team_box_scores(day, month, year, output_type=None, output_file_path=None, output_write_option=None,
                    json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.team_box_scores(day=day, month=month, year=year)
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidDate(day=day, month=month, year=year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=TEAM_BOX_SCORES_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=TEAM_BOX_SCORES_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def play_by_play(home_team, day, month, year, output_type=None, output_file_path=None, output_write_option=None,
                 json_options=None):
    try:
        http_service = HTTPService(parser=ParserService())
        values = http_service.play_by_play(home_team=home_team, day=day, month=month, year=year)
    except requests.exceptions.HTTPError as http_error:
        if http_error.response.status_code == requests.codes.not_found:
            raise InvalidDate(day=day, month=month, year=year)
        else:
            raise http_error
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=CSVWriter(
            column_names=PLAY_BY_PLAY_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=PLAY_BY_PLAY_COLUMN_NAMES)
        ),
        json_options=json_options,
    )


def search(term, output_type=None, output_file_path=None, output_write_option=None, json_options=None):
    http_service = HTTPService(parser=ParserService())
    values = http_service.search(term=term)
    return output(
        values=values,
        output_type=output_type,
        output_file_path=output_file_path,
        output_write_option=output_write_option,
        csv_writer=SearchResultsCSVWriter(
            column_names=SEARCH_RESULTS_COLUMN_NAMES,
            row_formatter=RowFormatter(data_field_names=SEARCH_RESULTS_COLUMN_NAMES),
        ),
        json_options=json_options,
    )



class InvalidDate(Exception):
    def __init__(self, day, month, year):
        message = "Date with year set to {year}, month set to {month}, and day set to {day} is invalid"\
            .format(
                year=year,
                month=month,
                day=day,
            )
        super().__init__(message)


class InvalidSeason(Exception):
    def __init__(self, season_end_year):
        message = "Season end year of {season_end_year} is invalid".format(season_end_year=season_end_year)
        super().__init__(message)


class InvalidPlayerAndSeason(Exception):
    def __init__(self, player_identifier, season_end_year):
        message = "Player with identifier \"{player_identifier}\" in season ending in {season_end_year} is invalid" \
            .format(player_identifier=player_identifier, season_end_year=season_end_year)
        super().__init__(message)



from basketball_reference_web_scraper.data import TEAM_ABBREVIATIONS_TO_TEAM, LOCATION_ABBREVIATIONS_TO_POSITION, OUTCOME_ABBREVIATIONS_TO_OUTCOME, TEAM_NAME_TO_TEAM, \
    POSITION_ABBREVIATIONS_TO_POSITION, LEAGUE_ABBREVIATIONS_TO_LEAGUE
from basketball_reference_web_scraper.parsers import PositionAbbreviationParser, TeamAbbreviationParser, \
    PlayerSeasonTotalsParser, TeamTotalsParser, LocationAbbreviationParser, OutcomeAbbreviationParser, \
    SecondsPlayedParser, PlayerBoxScoresParser, PlayerAdvancedSeasonTotalsParser, PeriodDetailsParser, \
    PeriodTimestampParser, ScoresParser, PlayByPlaysParser, TeamNameParser, ScheduledStartTimeParser, \
    ScheduledGamesParser, PlayerBoxScoreOutcomeParser, PlayerSeasonBoxScoresParser, SearchResultNameParser, \
    ResourceLocationParser, SearchResultsParser, LeagueAbbreviationParser, PlayerDataParser


class ParserService:
    PLAY_BY_PLAY_TIMESTAMP_FORMAT = "%M:%S.%f"
    PLAY_BY_PLAY_SCORES_REGEX = "(?P<away_team_score>[0-9]+)-(?P<home_team_score>[0-9]+)"
    SEARCH_RESULT_RESOURCE_LOCATION_REGEX = '(https?:\/\/www\.basketball-reference\.com\/)?(?P<resource_type>.+?(?=\/)).*\/(?P<resource_identifier>.+).html'

    def __init__(self):
        self.team_abbreviation_parser = TeamAbbreviationParser(abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM)
        self.league_abbreviation_parser=LeagueAbbreviationParser(abbreviations_to_league=LEAGUE_ABBREVIATIONS_TO_LEAGUE)
        self.location_abbreviation_parser = LocationAbbreviationParser(
            abbreviations_to_locations=LOCATION_ABBREVIATIONS_TO_POSITION,
        )
        self.outcome_abbreviation_parser = OutcomeAbbreviationParser(
            abbreviations_to_outcomes=OUTCOME_ABBREVIATIONS_TO_OUTCOME,
        )
        self.outcome_parser = PlayerBoxScoreOutcomeParser(outcome_abbreviation_parser=self.outcome_abbreviation_parser)
        self.period_details_parser = PeriodDetailsParser(regulation_periods_count=4)
        self.period_timestamp_parser = PeriodTimestampParser(timestamp_format=ParserService.PLAY_BY_PLAY_TIMESTAMP_FORMAT)
        self.position_abbreviation_parser = PositionAbbreviationParser(
            abbreviations_to_positions=POSITION_ABBREVIATIONS_TO_POSITION,
        )
        self.seconds_played_parser = SecondsPlayedParser()
        self.scores_parser = ScoresParser(scores_regex=ParserService.PLAY_BY_PLAY_SCORES_REGEX)
        self.search_result_name_parser = SearchResultNameParser()
        self.search_result_location_parser = ResourceLocationParser(
            resource_location_regex=ParserService.SEARCH_RESULT_RESOURCE_LOCATION_REGEX
        )
        self.team_name_parser = TeamNameParser(team_names_to_teams=TEAM_NAME_TO_TEAM)

        self.play_by_plays_parser = PlayByPlaysParser(
            period_details_parser=self.period_details_parser,
            period_timestamp_parser=self.period_timestamp_parser,
            scores_parser=self.scores_parser,
        )
        self.player_box_scores_parser = PlayerBoxScoresParser(
            team_abbreviation_parser=self.team_abbreviation_parser,
            location_abbreviation_parser=self.location_abbreviation_parser,
            outcome_abbreviation_parser=self.outcome_abbreviation_parser,
            seconds_played_parser=self.seconds_played_parser
        )
        self.player_data_parser = PlayerDataParser(
            search_result_location_parser=self.search_result_location_parser,
            league_abbreviation_parser=self.league_abbreviation_parser,
        )
        self.player_season_box_scores_parser = PlayerSeasonBoxScoresParser(
            team_abbreviation_parser=self.team_abbreviation_parser,
            location_abbreviation_parser=self.location_abbreviation_parser,
            outcome_parser=self.outcome_parser,
            seconds_played_parser=self.seconds_played_parser
        )
        self.player_season_totals_parser = PlayerSeasonTotalsParser(
            position_abbreviation_parser=self.position_abbreviation_parser,
            team_abbreviation_parser=self.team_abbreviation_parser,
        )
        self.player_advanced_season_totals_parser = PlayerAdvancedSeasonTotalsParser(
            team_abbreviation_parser=self.team_abbreviation_parser,
            position_abbreviation_parser=self.position_abbreviation_parser,
        )
        self.scheduled_start_time_parser = ScheduledStartTimeParser()
        self.scheduled_games_parser = ScheduledGamesParser(
            start_time_parser=self.scheduled_start_time_parser,
            team_name_parser=self.team_name_parser,
        )
        self.search_results_parser = SearchResultsParser(
            search_result_name_parser=self.search_result_name_parser,
            search_result_location_parser=self.search_result_location_parser,
            league_abbreviation_parser=self.league_abbreviation_parser,
        )
        self.team_totals_parser = TeamTotalsParser(team_abbreviation_parser=self.team_abbreviation_parser)

    def parse_play_by_plays(self, play_by_plays, away_team_name, home_team_name):
        return self.play_by_plays_parser.parse(
            play_by_plays=play_by_plays,
            away_team=self.team_name_parser.parse_team_name(team_name=away_team_name),
            home_team=self.team_name_parser.parse_team_name(team_name=home_team_name),
        )

    def parse_player_box_scores(self, box_scores):
        return self.player_box_scores_parser.parse(box_scores=box_scores)

    def parse_player_season_box_scores(self, box_scores):
        return self.player_season_box_scores_parser.parse(box_scores=box_scores)

    def parse_player_advanced_season_totals_parser(self, totals):
        return self.player_advanced_season_totals_parser.parse(totals=totals)

    def parse_player_season_totals(self, totals):
        return self.player_season_totals_parser.parse(totals=totals)

    def parse_scheduled_games(self, games):
        return self.scheduled_games_parser.parse_games(games)

    def parse_team_totals(self, first_team_totals, second_team_totals):
        return self.team_totals_parser.parse(first_team_totals=first_team_totals, second_team_totals=second_team_totals)

    def parse_player_search_results(self, nba_aba_baa_players):
        return self.search_results_parser.parse(nba_aba_baa_players=nba_aba_baa_players)

    def parse_player_data(self, player):
        return self.player_data_parser.parse(player=player)


from unittest import TestCase
from unittest.mock import patch

from basketball_reference_web_scraper.data import PeriodType
from basketball_reference_web_scraper.parsers import PeriodDetailsParser


class TestPeriodDetailsParser(TestCase):
    def test_is_overtime_is_true_when_period_count_is_greater_than_regulation_periods_count(self):
        parser = PeriodDetailsParser(regulation_periods_count=4)
        self.assertTrue(parser.is_overtime(period_count=5))

    def test_is_overtime_is_false_when_period_count_is_equal_to_regulation_periods_count(self):
        parser = PeriodDetailsParser(regulation_periods_count=4)
        self.assertFalse(parser.is_overtime(period_count=4))

    def test_is_overtime_is_false_when_period_count_is_less_than_regulation_periods_count(self):
        self.assertFalse(PeriodDetailsParser(regulation_periods_count=4).is_overtime(period_count=3))

    @patch.object(PeriodDetailsParser, 'is_overtime')
    def test_parse_period_number_is_period_count_in_regulation(self, mocked_is_overtime):
        mocked_is_overtime.return_value = False
        self.assertEqual(
            PeriodDetailsParser(regulation_periods_count=4).parse_period_number(period_count="some period count"),
            "some period count",
        )

    @patch.object(PeriodDetailsParser, 'is_overtime')
    def test_parse_period_number_is_difference_between_period_count_and_regulation_periods_count(self, mocked_is_overtime):
        mocked_is_overtime.return_value = True
        self.assertEqual(
            PeriodDetailsParser(regulation_periods_count=4).parse_period_number(period_count=5),
            1,
        )

    @patch.object(PeriodDetailsParser, 'is_overtime')
    def test_parse_period_type_is_overtime_when_in_overtime(self, mocked_is_overtime):
        mocked_is_overtime.return_value = True
        self.assertEqual(
            PeriodDetailsParser(regulation_periods_count=4).parse_period_type(period_count="some period count"),
            PeriodType.OVERTIME,
        )

    @patch.object(PeriodDetailsParser, 'is_overtime')
    def test_parse_period_type_is_quarter_when_in_regulation(self, mocked_is_overtime):
        mocked_is_overtime.return_value = False
        self.assertEqual(
            PeriodDetailsParser(regulation_periods_count=4).parse_period_type(period_count="some period count"),
            PeriodType.QUARTER,
        )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="basketball_reference_web_scraper",
    version="4.9.2",
    author="Jae Bradley",
    author_email="jae.b.bradley@gmail.com",
    license="MIT",
    description="A Basketball Reference client that generates data by scraping the website",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jaebradley/basketball_reference_web_scraper",
    packages=setuptools.find_packages(exclude=["tests"]),
    python_requires=">=3.4",
    install_requires=[
        "certifi==2018.10.15",
        "chardet==3.0.4",
        "idna==2.7",
        "lxml==4.5.1",
        "pytz==2018.6",
        "requests==2.20.0",
        "urllib3==1.24.3",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=[
        "NBA",
        "Basketball",
        "Basketball Reference",
        "basketball-reference.com",
    ],
)



from unittest import TestCase
from unittest.mock import patch, MagicMock

from requests import HTTPError, codes

from basketball_reference_web_scraper.client import season_schedule
from basketball_reference_web_scraper.errors import InvalidSeason
from basketball_reference_web_scraper.http_service import HTTPService


class TestSeasonSchedule(TestCase):
    @patch.object(HTTPService, "season_schedule")
    def test_not_found_raises_invalid_season(self, mocked_season_schedule):
        mocked_season_schedule.side_effect = HTTPError(response=MagicMock(status_code=codes.not_found))
        self.assertRaisesRegex(
            InvalidSeason,
            "Season end year of jaebaebae is invalid",
            season_schedule,
            season_end_year="jaebaebae")

    @patch.object(HTTPService, "season_schedule")
    def test_other_http_error_is_raised(self, mocked_season_schedule):
        mocked_season_schedule.side_effect = HTTPError(response=MagicMock(status_code=codes.internal_server_error))
        self.assertRaises(HTTPError, season_schedule, season_end_year=2018)



from unittest import TestCase

from basketball_reference_web_scraper.writers import WriteOptions


class TestWriteOptions(TestCase):
    def test_should_write_to_file_is_false_when_file_path_is_none(self):
        self.assertFalse(WriteOptions(file_path=None).should_write_to_file())

    def test_should_write_to_file_is_false_when_file_path_is_not_none_but_mode_is_none(self):
        self.assertFalse(WriteOptions(file_path="some file path", mode=None).should_write_to_file())

    def test_should_write_to_file_is_true_when_file_path_is_not_none_and_mode_is_not_none(self):
        self.assertTrue(WriteOptions(file_path="some file path", mode="some mode").should_write_to_file())

    def test_two_options_with_same_properties_are_equivalent(self):
        self.assertEqual(WriteOptions(), WriteOptions())

    def test_two_options_with_same_properties_except_file_path_are_not_equivalent(self):
        self.assertNotEqual(WriteOptions(file_path="some file path"), WriteOptions())

    def test_two_options_with_same_properties_except_mode_are_not_equivalent(self):
        self.assertNotEqual(WriteOptions(mode="some mode"), WriteOptions())

    def test_two_options_with_same_properties_except_custom_option_are_not_equivalent(self):
        self.assertNotEqual(WriteOptions(custom_options="some optoins"), WriteOptions())



import requests
from lxml import html

from basketball_reference_web_scraper.data import TEAM_TO_TEAM_ABBREVIATION, TeamTotal, PlayerData
from basketball_reference_web_scraper.errors import InvalidDate, InvalidPlayerAndSeason
from basketball_reference_web_scraper.html import DailyLeadersPage, PlayerSeasonBoxScoresPage, PlayerSeasonTotalTable, \
    PlayerAdvancedSeasonTotalsTable, PlayByPlayPage, SchedulePage, BoxScoresPage, DailyBoxScoresPage, SearchPage, \
    PlayerPage


class HTTPService:
    BASE_URL = 'https://www.basketball-reference.com'

    def __init__(self, parser):
        self.parser = parser

    def player_box_scores(self, day, month, year):
        url = '{BASE_URL}/friv/dailyleaders.cgi?month={month}&day={day}&year={year}'.format(
            BASE_URL=HTTPService.BASE_URL,
            day=day,
            month=month,
            year=year
        )

        response = requests.get(url=url, allow_redirects=False)

        response.raise_for_status()

        if response.status_code == requests.codes.ok:
            page = DailyLeadersPage(html=html.fromstring(response.content))
            return self.parser.parse_player_box_scores(box_scores=page.daily_leaders)

        raise InvalidDate(day=day, month=month, year=year)

    def regular_season_player_box_scores(self, player_identifier, season_end_year):
        # Makes assumption that basketball reference pattern of breaking out player pathing using first character of
        # surname can be derived from the fact that basketball reference also has a pattern of player identifiers
        # starting with first few characters of player's surname
        url = '{BASE_URL}/players/{player_surname_starting_character}/{player_identifier}/gamelog/{season_end_year}' \
            .format(
                BASE_URL=HTTPService.BASE_URL,
                player_surname_starting_character=player_identifier[0],
                player_identifier=player_identifier,
                season_end_year=season_end_year,
            )

        response = requests.get(url=url, allow_redirects=False)
        response.raise_for_status()

        page = PlayerSeasonBoxScoresPage(html=html.fromstring(response.content))
        if page.regular_season_box_scores_table is None:
            raise InvalidPlayerAndSeason(player_identifier=player_identifier, season_end_year=season_end_year)

        return self.parser.parse_player_season_box_scores(box_scores=page.regular_season_box_scores_table.rows)

    def play_by_play(self, home_team, day, month, year):
        add_0_if_needed = lambda s: "0" + s if len(s) == 1 else s

        # the hard-coded `0` in the url assumes we always take the first match of the given date and team.
        url = "{BASE_URL}/boxscores/pbp/{year}{month}{day}0{team_abbr}.html".format(
            BASE_URL=HTTPService.BASE_URL, year=year, month=add_0_if_needed(str(month)), day=add_0_if_needed(str(day)),
            team_abbr=TEAM_TO_TEAM_ABBREVIATION[home_team]
        )
        response = requests.get(url=url)
        response.raise_for_status()

        page = PlayByPlayPage(html=html.fromstring(response.content))

        return self.parser.parse_play_by_plays(
            play_by_plays=page.play_by_play_table.rows,
            away_team_name=page.away_team_name,
            home_team_name=page.home_team_name,
        )

    def players_advanced_season_totals(self, season_end_year, include_combined_values=False):
        url = '{BASE_URL}/leagues/NBA_{season_end_year}_advanced.html'.format(
            BASE_URL=HTTPService.BASE_URL,
            season_end_year=season_end_year,
        )

        response = requests.get(url=url)

        response.raise_for_status()

        table = PlayerAdvancedSeasonTotalsTable(html=html.fromstring(response.content))
        return self.parser.parse_player_advanced_season_totals_parser(totals=table.get_rows(include_combined_values))

    def players_season_totals(self, season_end_year):
        url = '{BASE_URL}/leagues/NBA_{season_end_year}_totals.html'.format(
            BASE_URL=HTTPService.BASE_URL,
            season_end_year=season_end_year,
        )

        response = requests.get(url=url)

        response.raise_for_status()

        table = PlayerSeasonTotalTable(html=html.fromstring(response.content))
        return self.parser.parse_player_season_totals(totals=table.rows)

    def schedule_for_month(self, url):
        response = requests.get(url=url)

        response.raise_for_status()

        page = SchedulePage(html=html.fromstring(html=response.content))
        return self.parser.parse_scheduled_games(games=page.rows)

    def season_schedule(self, season_end_year):
        url = '{BASE_URL}/leagues/NBA_{season_end_year}_games.html'.format(
            BASE_URL=HTTPService.BASE_URL,
            season_end_year=season_end_year
        )

        response = requests.get(url=url)

        response.raise_for_status()

        page = SchedulePage(html=html.fromstring(html=response.content))
        season_schedule_values = self.parser.parse_scheduled_games(games=page.rows)

        for month_url_path in page.other_months_schedule_urls:
            url = '{BASE_URL}{month_url_path}'.format(BASE_URL=HTTPService.BASE_URL, month_url_path=month_url_path)
            monthly_schedule = self.schedule_for_month(url=url)
            season_schedule_values.extend(monthly_schedule)

        return season_schedule_values

    def team_box_score(self, game_url_path):
        url = "{BASE_URL}/{game_url_path}".format(BASE_URL=HTTPService.BASE_URL, game_url_path=game_url_path)

        response = requests.get(url=url)

        response.raise_for_status()

        page = BoxScoresPage(html.fromstring(response.content))
        combined_team_totals = [
            TeamTotal(team_abbreviation=table.team_abbreviation, totals=table.team_totals)
            for table in page.basic_statistics_tables
        ]

        return self.parser.parse_team_totals(
            first_team_totals=combined_team_totals[0],
            second_team_totals=combined_team_totals[1],
        )

    def team_box_scores(self, day, month, year):
        url = "{BASE_URL}/boxscores/".format(BASE_URL=HTTPService.BASE_URL)

        response = requests.get(url=url, params={"day": day, "month": month, "year": year})

        response.raise_for_status()

        page = DailyBoxScoresPage(html=html.fromstring(response.content))

        return [
            box_score
            for game_url_path in page.game_url_paths
            for box_score in self.team_box_score(game_url_path=game_url_path)
        ]

    def search(self, term):
        response = requests.get(
            url="{BASE_URL}/search/search.fcgi".format(BASE_URL=HTTPService.BASE_URL),
            params={"search": term}
        )

        response.raise_for_status()

        player_results = []

        if response.url.startswith("{BASE_URL}/search/search.fcgi".format(BASE_URL=HTTPService.BASE_URL)):
            page = SearchPage(html=html.fromstring(response.content))
            parsed_results = self.parser.parse_player_search_results(nba_aba_baa_players=page.nba_aba_baa_players)
            player_results += parsed_results["players"]

            while page.nba_aba_baa_players_pagination_url is not None:
                response = requests.get(
                    url="{BASE_URL}/search/{pagination_url}".format(
                        BASE_URL=HTTPService.BASE_URL,
                        pagination_url=page.nba_aba_baa_players_pagination_url
                    )
                )

                response.raise_for_status()

                page = SearchPage(html=html.fromstring(response.content))

                parsed_results = self.parser.parse_player_search_results(nba_aba_baa_players=page.nba_aba_baa_players)
                player_results += parsed_results["players"]

        elif response.url.startswith("{BASE_URL}/players".format(BASE_URL=HTTPService.BASE_URL)):
            page = PlayerPage(html=html.fromstring(response.content))
            data = PlayerData(
                name=page.name,
                resource_location=response.url,
                league_abbreviations=set([row.league_abbreviation for row in page.totals_table.rows])
            )
            player_results += [self.parser.parse_player_data(player=data)]

        return {
            "players": player_results
        }



from unittest import TestCase

import requests
from lxml import html

from basketball_reference_web_scraper.data import TEAM_ABBREVIATIONS_TO_TEAM, LOCATION_ABBREVIATIONS_TO_POSITION, \
    OUTCOME_ABBREVIATIONS_TO_OUTCOME
from basketball_reference_web_scraper.data import Team, Outcome
from basketball_reference_web_scraper.html import DailyLeadersPage
from basketball_reference_web_scraper.parsers import TeamAbbreviationParser, \
    LocationAbbreviationParser, OutcomeAbbreviationParser, \
    SecondsPlayedParser, PlayerBoxScoresParser


class TestPlayerBoxScores(TestCase):
    def setUp(self):
        self.november_01_2006_daily_leaders = requests.get(
            'https://www.basketball-reference.com/friv/dailyleaders.fcgi?month=11&day=1&year=2006'
        ).text
        self.december_18_2015_daily_leaders = requests.get(
            'https://www.basketball-reference.com/friv/dailyleaders.fcgi?month=12&day=18&year=2015'
        ).text
        self.november_03_2003_daily_leaders = requests.get(
            'https://www.basketball-reference.com/friv/dailyleaders.fcgi?month=11&day=03&year=2003'
        ).text
        self.december_12_2017_daily_leaders = requests.get(
            'https://www.basketball-reference.com/friv/dailyleaders.fcgi?month=12&day=12&year=2017'
        ).text
        self.january_29_2017_daily_leaders = requests.get(
            'https://www.basketball-reference.com/friv/dailyleaders.fcgi?month=1&day=29&year=2017'
        ).text
        self.team_abbreviation_parser = TeamAbbreviationParser(
            abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM
        )
        self.location_abbreviation_parser = LocationAbbreviationParser(
            abbreviations_to_locations=LOCATION_ABBREVIATIONS_TO_POSITION
        )
        self.outcome_abbreviation_parser = OutcomeAbbreviationParser(
            abbreviations_to_outcomes=OUTCOME_ABBREVIATIONS_TO_OUTCOME
        )
        self.seconds_played_parser = SecondsPlayedParser()
        self.player_box_scores_parser = PlayerBoxScoresParser(
            team_abbreviation_parser=self.team_abbreviation_parser,
            location_abbreviation_parser=self.location_abbreviation_parser,
            outcome_abbreviation_parser=self.outcome_abbreviation_parser,
            seconds_played_parser=self.seconds_played_parser,
        )

    def test_box_scores_for_12_18_2015(self):
        page = DailyLeadersPage(html=html.fromstring(self.december_18_2015_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)
        self.assertEqual(len(parsed_box_score), 250)

    def test_box_scores_for_12_12_2017(self):
        page = DailyLeadersPage(html=html.fromstring(self.december_12_2017_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)
        self.assertEqual(len(parsed_box_score), 149)

    def test_parses_blank_value_for_andrew_bogut_on_12_12_2017(self):
        page = DailyLeadersPage(html=html.fromstring(self.december_12_2017_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)
        andrew_bogut = parsed_box_score[128]
        self.assertEqual(andrew_bogut["made_three_point_field_goals"], 0)
        self.assertEqual(andrew_bogut["attempted_three_point_field_goals"], 0)

    # Test for minutes played greater than or equal to 60 minutes
    def test_box_scores_for_01_01_2017(self):
        page = DailyLeadersPage(html=html.fromstring(self.january_29_2017_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)
        self.assertEqual(len(parsed_box_score), 170)

        first_box_score = parsed_box_score[0]

        self.assertEqual(first_box_score["slug"], "millspa01")
        self.assertEqual(first_box_score["name"], "Paul Millsap")
        self.assertEqual(first_box_score["team"], Team.ATLANTA_HAWKS)
        self.assertEqual(first_box_score["opponent"], Team.NEW_YORK_KNICKS)
        self.assertEqual(first_box_score["outcome"], Outcome.WIN)
        self.assertEqual(first_box_score["seconds_played"], 3607)
        self.assertEqual(first_box_score["made_field_goals"], 13)
        self.assertEqual(first_box_score["attempted_field_goals"], 29)
        self.assertEqual(first_box_score["made_three_point_field_goals"], 3)
        self.assertEqual(first_box_score["attempted_three_point_field_goals"], 8)
        self.assertEqual(first_box_score["made_free_throws"], 8)
        self.assertEqual(first_box_score["attempted_free_throws"], 10)
        self.assertEqual(first_box_score["offensive_rebounds"], 8)
        self.assertEqual(first_box_score["defensive_rebounds"], 11)
        self.assertEqual(first_box_score["assists"], 7)
        self.assertEqual(first_box_score["steals"], 1)
        self.assertEqual(first_box_score["blocks"], 0)
        self.assertEqual(first_box_score["turnovers"], 3)
        self.assertEqual(first_box_score["personal_fouls"], 4)
        self.assertEqual(first_box_score["game_score"], 31.3)

    def test_parses_new_orleans_hornets_for_box_scores_for_11_03_2003(self):
        page = DailyLeadersPage(html=html.fromstring(self.november_03_2003_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)

        self.assertEqual(len(parsed_box_score), 145)

        pj_brown = parsed_box_score[51]

        self.assertEqual(pj_brown["slug"], "brownpj01")
        self.assertEqual(pj_brown["name"], "P.J. Brown")
        self.assertEqual(pj_brown["team"], Team.NEW_ORLEANS_HORNETS)

    def test_parses_new_orleans_oklahoma_city_hornets_for_box_scores_for_11_01_2006(self):
        page = DailyLeadersPage(html=html.fromstring(self.november_01_2006_daily_leaders))
        parsed_box_score = self.player_box_scores_parser.parse(page.daily_leaders)
        self.assertEqual(len(parsed_box_score), 272)

        chris_paul = parsed_box_score[10]

        self.assertEqual(chris_paul["slug"], "paulch01")
        self.assertEqual(chris_paul["name"], "Chris Paul")
        self.assertEqual(chris_paul["team"], Team.NEW_ORLEANS_OKLAHOMA_CITY_HORNETS)



from unittest import TestCase
from unittest.mock import MagicMock, patch

from requests import HTTPError, codes

from basketball_reference_web_scraper.client import regular_season_player_box_scores
from basketball_reference_web_scraper.errors import InvalidPlayerAndSeason
from basketball_reference_web_scraper.http_service import HTTPService


class TestPlayerRegularSeasonBoxScores(TestCase):
    @patch.object(HTTPService, "regular_season_player_box_scores")
    def test_raises_exception_for_500_response(self, mocked_regular_season_player_box_scores):
        mocked_regular_season_player_box_scores.side_effect = HTTPError(
            response=MagicMock(status_code=codes.internal_server_error)
        )
        self.assertRaises(InvalidPlayerAndSeason, regular_season_player_box_scores, 'Mock Player', 2000)

    @patch.object(HTTPService, "regular_season_player_box_scores")
    def test_raises_exception_for_404_response(self, mocked_regular_season_player_box_scores):
        mocked_regular_season_player_box_scores.side_effect = HTTPError(
            response=MagicMock(status_code=codes.not_found)
        )
        self.assertRaises(InvalidPlayerAndSeason, regular_season_player_box_scores, 'Mock Player', 2000)

    @patch.object(HTTPService, "regular_season_player_box_scores")
    def test_raises_non_500_http_error(self, mocked_regular_season_player_box_scores):
        mocked_regular_season_player_box_scores.side_effect = HTTPError(response=MagicMock(status_code=codes.bad_request))
        self.assertRaises(HTTPError, regular_season_player_box_scores, 'Mock Player', 2000)
